@@ -7,7 +7,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -17,7 +17,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #ifndef	_ALBH_
@@ -26,6 +27,13 @@
 #include "seq.h"
 #include "simmtx.h"
 
+enum TraceBackDir {
+	DEAD, RSRV, DIAG, NEWD, VERT, SLA1, SLA2, VERL, 
+	HORI, HOR1, HOR2, HORL, NEWV, NEWH,
+	SPIN=16, SPJC=32, SPF2=64, SPJCI=SPJC + SPIN,
+	SPALL=SPIN+SPJC+SPF2
+};
+
 static	const	int	NOL = 3;
 static	const	int	NOD = 2 * NOL - 1;
 static	const	int	INTR = 2;
@@ -41,6 +49,7 @@ static	const	float	def_alprm2z = 2.;
 static	const	int	end_of_ulk = INT_MAX - 2;
 static	const	int	dir2nod[16] = 
 	{-1, -1, 0, 0, 2, 2, 2, 4, 1, 1, 1, 3, 2, 1, -1, -1};
+static	const	int	nod2dir[5] = {DIAG, HORI, VERT, HORL, VERL};
 static	const	float	defSss[3] = {0.3, 0.50, 0.7};
 static	const	int	N_Out_Modes = 16;
 static	const	int	NCAND = 4;
@@ -49,13 +58,6 @@ static	const	CHAR	e1_psp = 1;
 static	const	CHAR	e2_psp = 2;
 static	const	CHAR	e_psp = e1_psp + e2_psp;
 
-enum TraceBackDir {
-	DEAD, RSRV, DIAG, NEWD, VERT, SLA1, SLA2, VERL, 
-	HORI, HOR1, HOR2, HORL, NEWV, NEWH,
-	SPIN=16, SPJC=32, SPF2=64, SPJCI=SPJC + SPIN,
-	SPALL=SPIN+SPJC+SPF2
-};
-
 enum DistMes {QFdiv, QFidn, QCdiv, QCidn, QJuCa, Qpamd, NJuCa};
 enum OutFm {GFF_FORM, ALN_FORM, PWA_FORM, BED_FORM, EXN_FORM, ITN_FORM, 
 	CDS_FORM, AAS_FORM, CIG_FORM, VLG_FORM, SAM_FORM, BAM_FORM,
@@ -288,8 +290,14 @@ const	Simmtx*	simmtx;
 	VTYPE	UnpPenalty3(int i) const {
 	    int	d = i / 3;
 	    VTYPE	unp = d * BasicGEP;
-	    if (i <= codonk1) return unp;
-	    return (unp - diffu * (d - alprm.k1) + (i % 3? ExtraGOP: 0));
+	    VTYPE	egop = 0;
+	    switch (i % 3) {
+		case 1: egop = GapE1; break;
+		case 2: egop = GapE2; break;
+		default: break;
+	    }
+	    if (i <= codonk1) return (unp + egop);
+	    return (unp - diffu * (d - alprm.k1) + egop);
 	}
 	VTYPE	GapExtPen3(int i) const {
 	    return (i > codonk1)? LongGEP: BasicGEP;
@@ -331,16 +339,18 @@ extern	VTYPE	selfAlnScr(const Seq* sd, const Simmtx* sm);
 extern	int	prePwd(int molc, bool use_mdm = false);
 extern	int	prePwd(const Seq* sd, bool use_mdm = false);
 extern	int	prePwd(const Seq** seqs, bool use_mdm = false);
-extern	void	stripe(const Seq* seqs[], WINDOW* wdw, int sh);
-extern	void	stripe31(const Seq* seqs[], WINDOW* wdw, int shld);
+extern	void	stripe(const Seq* seqs[], WINDOW* wdw, int sh, 
+	const int& cmode = 3);
+extern	void	stripe31(const Seq* seqs[], WINDOW* wdw, int shld,
+	const int& cmode = 3);
 extern	Seq*	synthseq(Seq* c, const Seq* a, const Seq* b, const SKL* skl);
 
 extern	VTYPE	HomScoreB_ng(const Seq* seqs[], const PwdB* pwd);
 extern	VTYPE	HomScoreH_ng(const Seq* seqs[], const PwdB* pwd);
 extern	VTYPE	HomScoreS_ng(const Seq* seqs[], const PwdB* pwd);
-extern	SKL*	alignB_ng(const Seq* seqs[], const PwdB* pwd, VTYPE* scr);
-extern	SKL*	alignH_ng(const Seq* seqs[], const PwdB* pwd, VTYPE* scr);
-extern	SKL*	alignS_ng(Seq* seqs[], const PwdB* pwd, VTYPE* scr, int ori = 3);
+extern	SKL*	alignB_ng(const Seq* seqs[], const PwdB* pwd, Gsinfo* gsi);
+extern	SKL*	alignH_ng(const Seq* seqs[], const PwdB* pwd, Gsinfo* gsi);
+extern	SKL*	alignS_ng(Seq* seqs[], const PwdB* pwd, Gsinfo* gsi, int ori = 3);
 extern	SKL*	nogap_skl(const Seq* a, const Seq* b = 0);
 extern	VTYPE	skl_rngB_ng(const Seq* seqs[], Gsinfo* gsi, const PwdB* pwd);
 extern	VTYPE	skl_rngH_ng(const Seq* seqs[], Gsinfo* gsi, const PwdB* pwd);
