@@ -6,7 +6,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -16,7 +16,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include "aln.h"
@@ -52,6 +53,7 @@ static	SHORT	Ncand = NCAND2PHS;	// max number of candidates to pass to 2nd phase
 static	const	char*	ConvPat = 0;
 static	const	char*	sBitPat = 0;
 static	float	aaafact = 1;
+static	float	drop_rate = 1;
 static	SHORT	MinSigpr = 3;
 static	INT	MaxNref = 16;
 static	int	AllRef = 0;
@@ -70,6 +72,7 @@ static	float	ild_up_quantile = 0.996;
 static	BLKTYPE	MaxBlock = 0;
 static	BLKTYPE	ExtBlock = 0;
 static	BLKTYPE	ExtBlockL = 0;
+static	double	cfact = 0.75;
 
 static	void	setupbitpat(int molc, size_t gnmsz);
 static	bool	newer(char* str, const char** argv);
@@ -108,6 +111,10 @@ const	char*	val = ps + 1;
 	    case 'M':	// Maximum gene length
 		if (*val) OutPrm.MaxOut2 = atoi(val);
 		break;
+	    case 'P':	// paralog chain score drop rate
+		if (*val) drop_rate = atof(val);
+		if (drop_rate > 1) drop_rate /= 100;
+		break;
 	    case 'Q':	// Maximum gene length
 		if (*val) ild_up_quantile = atof(val);
 		break;
@@ -122,8 +129,8 @@ const	char*	val = ps + 1;
 	    case 'b':	// block length
 		if (*val) wcp.blklen = ktoi(val);
 		break;
-	    case 'c':	// factor for hash mergin
-		if (*val) hfact = atof(val);
+	    case 'c':	// exponet to Nbitpat
+		if (*val) cfact = atof(val);
 		break;
 	    case 'd':	// N-best all hit block score
 		if (*val) Nascr = atoi(val);
@@ -140,6 +147,9 @@ const	char*	val = ps + 1;
 	    case 'h':	// bias of phase1 threshold
 		if (*val) RbsBias = atof(val);
 		break;
+	    case 'j':	// factor for hash mergin
+		if (*val) hfact = atof(val);
+		break;
 	    case 'k':	// tuple size
 		if (*val) wcp.Ktuple = atoi(val);
 		break;
@@ -186,22 +196,31 @@ const	char*	val = ps + 1;
 MakeDbs::MakeDbs(const char* dbn, int molc)
 	: isaa(molc == PROTEIN)
 {
+	if (WriteFile) {
+	    write_path = new char[strlen(WriteFile) + 1];
+	    strcpy(write_path, WriteFile);
+	    char* sls = strrchr(write_path, '/');
+	    if (sls) {
+		*sls = 0;
+		OutPrm.out_file = write_path;
+	    }
+	}
 	entryprv[0] = '\0';
 	dbname = strrealloc(0, dbn);
 #if USE_ZLIB
 	if (OutPrm.gzipped) {
-	    gzseq = gzopenpbe("", dbname, SGZ_EXT, "w", 2);
-	    gzidx = gzopenpbe("", dbname, IDZ_EXT, "w", 2);
-	    gzent = gzopenpbe("", dbname, ENZ_EXT, "w", 2);
+	    gzseq = gzopenpbe(OutPrm.out_file, dbname, SGZ_EXT, "w", 2);
+	    gzidx = gzopenpbe(OutPrm.out_file, dbname, IDZ_EXT, "w", 2);
+	    gzent = gzopenpbe(OutPrm.out_file, dbname, ENZ_EXT, "w", 2);
 	} else
 #endif
 	{
-	    fseq = fopenpbe("", dbname, SEQ_EXT, "w", 2);
-	    fidx = fopenpbe("", dbname, IDX_EXT, "w+", 2);
-	    fent = fopenpbe("", dbname, ENT_EXT, "w+", 2);
+	    fseq = fopenpbe(OutPrm.out_file, dbname, SEQ_EXT, "w", 2);
+	    fidx = fopenpbe(OutPrm.out_file, dbname, IDX_EXT, "w+", 2);
+	    fent = fopenpbe(OutPrm.out_file, dbname, ENT_EXT, "w+", 2);
 	}
 	vclear(&rec);
-	fgrp = fopenpbe("", dbname, GRP_EXT, "w", 2);
+	fgrp = fopenpbe(OutPrm.out_file, dbname, GRP_EXT, "w", 2);
 }
 
 static	DbsRec*	rbuf;
@@ -217,24 +236,43 @@ void MakeDbs::read_ent(file_t fd)
 {
 	size_t	flen = ftell(fd);
 	cbuf = new char[flen];
+#if USE_ZLIB
 	if (gzent) {
 	    fclose(fd);
-	    gzent = gzopenpbe("", dbname, ENZ_EXT, "r", 2);
-	} else	rewind(fd);
+	    gzent = gzopenpbe(OutPrm.out_file, dbname, ENZ_EXT, "r", 2);
+	    if (fread(cbuf, sizeof(char), flen, gzent) != flen)
+		fatal("Corrupted entry file !\n");
+	} else {
+	    rewind(fd);
+	    if (fread(cbuf, sizeof(char), flen, fd) != flen)
+		fatal("Corrupted entry file !\n");
+	}
+#else
+	rewind(fd);
 	if (fread(cbuf, sizeof(char), flen, fd) != flen)
 	    fatal("Corrupted entry file !\n");
+#endif
 }
 
 template <typename file_t>
 void MakeDbs::read_idx(file_t fd)
 {
+#if USE_ZLIB
 	if (gzidx) {
 	    fclose(fd);
-	    gzidx = gzopenpbe("", dbname, IDZ_EXT, "r", 2);
-	} else
+	    gzidx = gzopenpbe(OutPrm.out_file, dbname, IDZ_EXT, "r", 2);
+	    if (fread(rbuf, sizeof(DbsRec), recnbr, gzidx) != recnbr)
+		fatal("Corrupted index file !\n");
+	} else {
 	    rewind(fd);
+	    if (fread(rbuf, sizeof(DbsRec), recnbr, fd) != recnbr)
+		fatal("Corrupted index file !\n");
+	}
+#else
+	rewind(fd);
 	if (fread(rbuf, sizeof(DbsRec), recnbr, fd) != recnbr)
 	    fatal("Corrupted index file !\n");
+#endif
 }
 
 template <typename file_t>
@@ -256,27 +294,33 @@ void MakeDbs::write_odr(file_t fd, const INT* order, const char* str)
 void MakeDbs::mkidx()
 {
 	if (!cridxf) return;		// has been sorted
+	char	str[LINE_MAX];
+	FILE*	fodr = 0;
+#if USE_ZLIB
 	if (gzent)	read_ent(gzent);
 	else 		read_ent(fent);
 	rbuf = new DbsRec[recnbr];
 	if (gzidx)	read_idx(gzidx);
 	else		read_idx(fidx);
+#else
+	read_ent(fent);
+	rbuf = new DbsRec[recnbr];
+	read_idx(fidx);
+#endif
 	INT*	order = new INT[recnbr];
 	for (INT i = 0; i < recnbr; ++i) order[i] = i;
 	qsort((UPTR) order, recnbr, sizeof(INT), (CMPF) cmpkey);
-	char	str[LINE_MAX];
-	FILE*	fodr = 0;
 #if USE_ZLIB
 	gzFile	gzodr = 0;
 	if (OutPrm.gzipped)
-	    gzodr = gzopenpbe("", dbname, ODZ_EXT, "w", 2, str);
+	    gzodr = gzopenpbe(OutPrm.out_file, dbname, ODZ_EXT, "w", 2, str);
 	else
-	    fodr = fopenpbe("", dbname, ODR_EXT, "w", 2, str);
+	    fodr = fopenpbe(OutPrm.out_file, dbname, ODR_EXT, "w", 2, str);
 	if (fodr)	write_odr(fodr, order, str);
 	else		write_odr(gzodr, order, str);
 #else
-	fodr = fopenpbe("", dbname, ODR_EXT, "w", 2, str);
-	write_odr(fodr, str);
+	fodr = fopenpbe(OutPrm.out_file, dbname, ODR_EXT, "w", 2, str);
+	write_odr(fodr, order, str);
 #endif
 	delete[] cbuf;
 	delete[] rbuf;
@@ -288,14 +332,23 @@ int MakeDbs::write_recrd(file_t fd, int c)
 { 
 	putseq(SEQ_DELIM);
 	if (rec.seqlen) {
+#if USE_ZLIB
 	    if (gzidx)	write_idx(gzidx);
 	    else	write_idx(fidx);
+#else
+	    write_idx(fidx);
+#endif
 	}
 	if (c != EOF) {
+#if USE_ZLIB
 	    if (gzent)	rec.entptr = ftell(gzent);
 	    else	rec.entptr = ftell(fent);
 	    if (gzseq)	rec.seqptr = ftell(gzseq);
 	    else	rec.seqptr = ftell(fseq);
+#else
+	    rec.entptr = ftell(fent);
+	    rec.seqptr = ftell(fseq);
+#endif
 	    char*	ps = entrystr;
 	    char*	pe = ps;
 	    char*	pb = ps;
@@ -311,11 +364,15 @@ int MakeDbs::write_recrd(file_t fd, int c)
 	    else	*ps = '\0';
 	    if (wordcmp(pe, entryprv) < 0) cridxf = true;
 	    strcpy(entryprv, pe);
+#if USE_ZLIB
 	    if (gzent) {
 		fputs(pe, gzent); fputc('\0', gzent);
 	    } else {
 		fputs(pe, fent); fputc('\0', fent);
 	    }
+#else
+	    fputs(pe, fent); fputc('\0', fent);
+#endif
 	}
 	rec.seqlen = b = 0;
 	return (c);
@@ -492,15 +549,10 @@ MakeBlk::MakeBlk(const Seq* sq, DbsDt* dd, MakeDbs* mdbs, bool mk_blk) :
 	if (!molc && dd) molc = dd->curdb->defmolc;
 	defcode = setSeqCode(0, isaa? PROTEIN: DNA);
 	if (isaa) bias = ALA - 1;
-	INT	nwd = (wcp.blklen / wcp.Nshift + 1) * wcp.Nbitpat;
-	if (istron) {
-	    prepacomp();
-	    nwd *= 2;
-	}
-#if M_THREAD
-#endif	// M_THREAD
+	if (istron) prepacomp();
+
 	if (dd) {
-	    char*	cvt = isaa? acodon: nucl;
+const	    char*	cvt = isaa? acodon: nucl;
 	    s2r = new CHAR[defcode->max_code];
 	    s2r[nil_code] = s2r[gap_code] = Nalpha + 1;
 	    s2r[defcode->amb_code] = Nalpha;
@@ -625,25 +677,17 @@ static  const char* wfmt =
 
 static void setupbitpat(int molc, size_t gnmsz)
 {
-	if (molc == PROTEIN || molc == TRON) {
-	    BlkWcPrm&	wcp_t = algmode.crs == 2? wcp_ax: wcp_af;
-	    if (wcp.Nalpha == 0) wcp.Nalpha = wcp_t.Nalpha;
-	    if (wcp.Ktuple == 0 && !gnmsz) wcp.Ktuple = wcp_t.Ktuple;
-	    if (wcp.Bitpat2 == 1) wcp.Bitpat2 = wcp_t.Bitpat2;
-	    if (wcp.BitPat == 1) wcp.BitPat = wcp_t.BitPat;
-	    if (wcp.blklen == 0 && !gnmsz) wcp.blklen = wcp_t.blklen;
-	    if (wcp.Nbitpat == 0) wcp.Nbitpat = wcp_t.Nbitpat;
-	    if (wcp.afact == 0) wcp.afact = wcp_t.afact;
-	} else {
-	    BlkWcPrm&	wcp_t = algmode.crs == 2? wcp_cx: wcp_cf;
-	    if (wcp.Nalpha == 0) wcp.Nalpha = wcp_t.Nalpha;
-	    if (wcp.Ktuple == 0 && !gnmsz) wcp.Ktuple = wcp_t.Ktuple;
-	    if (wcp.Bitpat2 == 1) wcp.Bitpat2 = wcp_t.Bitpat2;
-	    if (wcp.BitPat == 1) wcp.BitPat = wcp_t.BitPat;
-	    if (wcp.blklen == 0 && !gnmsz) wcp.blklen = wcp_t.blklen;
-	    if (wcp.Nbitpat == 0) wcp.Nbitpat = wcp_t.Nbitpat;
-	    if (wcp.afact == 0) wcp.afact = wcp_t.afact;
-	}
+	BlkWcPrm&	wcp_t = (molc == PROTEIN || molc == TRON)?
+	    (algmode.crs == 2? wcp_ax: wcp_af):
+	    (algmode.crs == 2? wcp_cx: wcp_cf);
+	if (wcp.Nalpha == 0) wcp.Nalpha = wcp_t.Nalpha;
+	if (wcp.Ktuple == 0 && !gnmsz) wcp.Ktuple = wcp_t.Ktuple;
+	if (wcp.Bitpat2 == 1) wcp.Bitpat2 = wcp_t.Bitpat2;
+	if (wcp.BitPat == 1) wcp.BitPat = wcp_t.BitPat;
+	if (wcp.blklen == 0 && !gnmsz) wcp.blklen = wcp_t.blklen;
+	if (wcp.Nbitpat == 0) wcp.Nbitpat = wcp_t.Nbitpat;
+	if (wcp.afact == 0) wcp.afact = wcp_t.afact;
+
 	if (gnmsz && !(wcp.Ktuple && wcp.blklen && wcp.MaxGene)) {
 	    if (!wcp.blklen) {
 		wcp.blklen = int(sqrt(double(gnmsz)));
@@ -1147,20 +1191,20 @@ const	char**	seqdb = argv;
 	for (int ac = 0; ac++ < argc && *seqdb; ++seqdb) {
 	    if (mkdbs) mkdbs->wrtgrp(*argv);
 	    bool	gz = is_gz(*seqdb);
-	    if (gz) {
 #if USE_ZLIB
+	    if (gz) {
 		gzFile	gzfd = gzopen(*seqdb, "rb");
 		if (gzfd)
 		    scan_genome(gzfd, tcount);
 		else	fatal("%s not found!\n", *seqdb);
-#else
-		fatal(gz_unsupport);
-#endif
 	    } else {
 		FILE*	fd = fopen(*seqdb, "r");
 		if (!fd) fatal("%s not found!\n", *seqdb);
 		scan_genome(fd, tcount);
 	    }
+#else
+	    if (gz) fatal(gz_unsupport);
+#endif
 	}
 	if (mkdbs) mkdbs->wrtgrp("E_O_F");
 	if (!mkblk) return;
@@ -1962,7 +2006,7 @@ Seq* SrchBlk::setgnmrng(const BPAIR* wrkbp)
 	INT	x = (wrkbp->zl? wrkbp->lb - wrkbp->zl: 0) * wcp.blklen;
 	INT	y = ((wrkbp->zl? wrkbp->rb - wrkbp->zl: 0) + 1) * wcp.blklen;
 	char	str[LINE_MAX];
-	sprintf(str, "Dbs%d %d %d", wrkbp->chr, x + 1, y);
+	snprintf(str, LINE_MAX, "Dbs%d %d %d", wrkbp->chr, x + 1, y);
 	Seq*	sd = (*curgr)->getdbseq(dbf, str, wrkbp->chr);
 	if (wrkbp->rvs) sd->comrev();
 	return (sd);
@@ -2274,10 +2318,10 @@ bool extend_gene_rng(Seq* sqs[], const PwdB* pwd, DbsDt* dbf)
 
 	    char	str[LINE_MAX];
 	    if (rvs)
-		sprintf(str, "$%s %d %d <", (*b->sname)[0],
+		snprintf(str, LINE_MAX, "$%s %d %d <", (*b->sname)[0],
 		    b->SiteNo(grng.right), b->SiteNo(grng.left));
 	    else
-		sprintf(str, "$%s %d %d", (*b->sname)[0],    
+		snprintf(str, LINE_MAX, "$%s %d %d", (*b->sname)[0],    
 		    b->SiteNo(grng.left), b->SiteNo(grng.right));
 	    Seq*    tmp = new Seq(1, abs(grng.right - grng.left));
 	    swap(b, tmp);
@@ -2304,7 +2348,6 @@ int SrchBlk::FindHsp(BPAIR* wrkbp)
 const	INT	intr = (*gener)->inex.intr;
 const	int	sr = chrsize(wrkbp->chr);
 const	bool	rvs = wrkbp->rvs;
-const	float	log_2[4] = {0.f, 1.0f, 1.585f, 2.0f};
 
 	wrkbp->jscr = 0;
 	Seq*	cursd = 0;
@@ -2340,10 +2383,10 @@ retry:
 		float	x = wlu->scr;
 		wlu->scr = int(x * qlen / wlu->tlen);
 	    }
-	    wlu->scr += IntronPrm.sip * log_2[std::min(wlu->num - 1, 3)] ;
 	    if (wlu->scr >= lcritjscr) {
 		++nbetter;
-		lcritjscr = wlu->scr - vthr;
+		if (drop_rate < 1) lcritjscr = wlu->scr * drop_rate;
+		else	lcritjscr = wlu->scr - vthr;
 		JUXT*	wjxt = wlu->jxt;
 	 	if (lend.jx > wjxt->jx) lend = *wjxt;
 	 	wjxt += wlu->num - 1;
@@ -2416,7 +2459,7 @@ retry:
 		wrkbp->db = min(wrkbp->rb + ExtBlock, wrkbp->zr);
 	    }
 	}
-	if (retry_no++ < NoRetry && 
+	if (pwd->DvsP == 1 && retry_no++ < NoRetry && 
 		(wrkbp->lb != orgbp.lb || wrkbp->rb != orgbp.rb)) {
 	    orgbp = *wrkbp;
 	    goto retry;
@@ -2487,6 +2530,7 @@ retry:
 	    }
 	    if (curgr - gener >= OutPrm.MaxOut - 1) {
 		critjscr = gener[OutPrm.MaxOut - 1]->jscr - vthr;
+//		critjscr = gener[OutPrm.MaxOut - 1]->jscr * 3 / 4;
 		if (critjscr < 0) critjscr = 0;
 	    }
 	    if (curgr < lstgr) ++curgr;
@@ -2779,6 +2823,7 @@ Qwords::Qwords(int k, int nc, CHAR* ct, ContBlk* pwc, Bitpat** bp, Seq* a) :
 	xx = new int[kk];
 	front = new BLKTYPE[kk];
 	endss = new CHAR*[kk];
+	if (kk > 1) app_c = pow((double) wcp.Nbitpat, cfact);
 	if (a) reset(a);
 }
 
@@ -2803,6 +2848,7 @@ int Qwords::querywords(const CHAR* ss)
 	} else {
 	    INT*	w = ww;
 	    int*	x = xx;
+	    vset(ww, wcp.TabSize, kk);
 	    for (int k = 0; k < kk; ++k, ++w, ++x) {	// multiple bit patterns
 		if (ss >= endss[k]) break;
 		*w = *x = 0;
@@ -2820,13 +2866,13 @@ int Qwords::querywords(const CHAR* ss)
 	    w = ww;
 	    x = xx;
 	    for (int k = 0; k < kk; ++k, ++w, ++x) {
-		if (wc->wscr[*w] < 0) *x = -1;		// absent
+		if (*w >= wcp.TabSize || wc->wscr[*w] < 0) *x = -1;		// absent
 		else if (!*x && wc->blkp[*w]) {		// not amb or ubiquitous
 		    ++c;
 		    wdscr += wc->wscr[*w];
 		}
 	    }
-	    if (c) return (wdscr / c);			// find hit
+	    if (c) return (int(double(wdscr) / app_c));	// find hit
 	}
 	return (-1);					// no hit or bad query
 }
@@ -2855,6 +2901,7 @@ int Qwords::querywords(const CHAR* ss, int d, bool rvs)
 	    int	k = 0;
 	    INT*	w = ww;
 	    int*	x = xx;
+	    vset(ww, wcp.TabSize, kk);
 	    for ( ; k < kk; ++k, ++w, ++x) {		// multiple bit patterns
 		if (ss >= endss[k]) break;
 		*w = *x = 0;
@@ -2877,13 +2924,13 @@ int Qwords::querywords(const CHAR* ss, int d, bool rvs)
 	    int	c = 0;
 	    int	wdscr = 0;
 	    for (k = 0, w = ww, x = xx; k < kk; ++k, ++w, ++x) {
-		if (wc->wscr[*w] < 0) *x = -1;		// absent
+		if (*w >= wcp.TabSize || wc->wscr[*w] < 0) *x = -1;		// absent
 		else if (!*x && wc->blkp[*w]) {		// amb or ubiquitous
 		    ++c;
 		    wdscr += wc->wscr[*w];
 		}
 	    }
-	    if (c) return (wdscr / c);			// find hit
+	    if (c) return (int(double(wdscr) / app_c));	// find hit
 	}
 	return (-1);					// no hit or bad query
 }
@@ -2970,6 +3017,7 @@ const		    CHAR*	ss = *ws;
 		    if (wdscr == 0) {q = 1; continue;}	// ubiquitous
 		    qwd.init_mrglist();
 		    ++p; q = 0;
+//		    if (nmmc == 0) wdscr = wdscr * 5 / 4;	// end bonus
 		    cscr += wdscr;
 		    while (BLKTYPE blk = qwd.next_mrglist()) {
 			KVpair<INT, int>*	h = hh.incr(blk);
@@ -3040,12 +3088,12 @@ const		    CHAR*	ss = *ws;
 
 INT SrchBlk::bestref(Seq* sqs[], KVpair<INT, int>* sh, int n)
 {
-	Seq*&	a = sqs[1];
 	Wilip**	wl = new Wilip*[n];
 	GeneRng	gr;
 	Mfile	mfd(sizeof(GeneRng));
 
 	swap(sqs[0], sqs[1]);
+	Seq*&	a = sqs[0];
 	for (int i = 0; i < n; ++i, ++sh) {
 	    if (!sh->val) continue;
 	    setaaseq(a, sh->key);
@@ -3139,9 +3187,8 @@ int SrchBlk::findh(Seq** sqs)
 	if (!orf) return (ERROR);
 	Dhash<INT, int>	shash(2 * MaxNref, 0);
 	Qwords	qwd(kk,  DRNA, ConvTab, pbwc, bpp);
-	INT	norf = 0;
 	int	c;
-	for (ORF* wrf = orf; wrf->len; ++norf, ++wrf) {
+	for (ORF* wrf = orf; wrf->len; ++wrf) {
 	  bool	meet = false;
 	  b->translate(a, *wrf);
 	  if (a->right - a->left - (wcp.Nshift + bpp[0]->width) < 1) continue;
