@@ -7,7 +7,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -17,7 +17,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include "aln.h"
@@ -65,7 +66,7 @@ Cip_score::Cip_score(const Seq* sd)
 {
 	PfqItr	api(sd, sd->left);
 	if (api.size()) {
-	    cip_hash = new Dhash<int, VTYPE>(api.pfqnum);
+	    cip_hash = new Dhash<int, VTYPE>(api.pfqnum, 0);
 	    for ( ; !api.end(); ++api) {
 #if USE_WEIGHT
 		VTYPE	val = SpbFact * api.wfq->dns;
@@ -435,7 +436,7 @@ const	SigII*	sgi = sd->sigII;
 		int abc[3];
 		float	d = 100. * sgi->eij_dist(i, j, abc);
 		if (dmx) {
-		    sprintf(str, " %7.3f", d);
+		    snprintf(str, MAXL, " %7.3f", d);
 		    if (clm >= MaxClm) {
 			fputc('\n', fd);
 			clm = 0;
@@ -864,7 +865,7 @@ void Gsinfo::BoundaryInf(const Seq* sd) const
 	}
 const	EISCR*	wrng = eijnc->begin();
 const	EISCR*	trng = wrng + noeij - 1;
-const	EXIN*	bb = sd->exin->data;
+const	SGPT6*	bb = sd->exin->data_p;
 	int	len = -wrng->left;
 	int	clen = 0;
 	double	tscr = 0.;
@@ -1224,12 +1225,12 @@ void Vulgar::postproc()
 	}
 }
 
-Eijnc::Eijnc(bool que)
+Eijnc::Eijnc(bool que) : qsize(que? alprm2.jneibr: 0)
 {
 	emfd = new Mfile(sizeof(EISCR));
 	if (que) {
-	    fstque = new FSTAT[alprm2.jneibr];
-	    vclear(fstque, alprm2.jneibr);
+	    fstque = new FSTAT[qsize];
+	    vclear(fstque, qsize);
 	}
 }
 
@@ -1249,18 +1250,16 @@ void Eijnc::store(EISCR& rbuf, FSTAT& now, FSTAT& prv, bool nearjnc)
 	rbuf.mmc3 = (int) (now.mmc - fstque[q].mmc);
 	rbuf.unp3 = (int) (now.unp - fstque[q].unp);
 	rbuf.gap3 = (int) (now.gap - fstque[q].gap);
-	vclear(fstque, alprm2.jneibr);
-	q = 0;
 }
 
-void Eijnc::shift(EISCR& rbuf, FSTAT& now, FSTAT& prv, bool nearjnc)
+void Eijnc::shift(EISCR& rbuf, FSTAT& now, bool nearjnc)
 {
 	if (nearjnc) {
-	    rbuf.mch5 = (int) (now.mch - prv.mch);
-	    rbuf.mmc5 = (int) (now.mmc - prv.mmc);
-	    rbuf.unp5 = (int) (now.unp - prv.unp);
-	    rbuf.gap5 = (int) (now.gap - prv.gap);
+	    rbuf.mch5 = (int) (now.mch - fstque[q].mch);
+	    rbuf.mmc5 = (int) (now.mmc - fstque[q].mmc);
+	    rbuf.unp5 = (int) (now.unp - fstque[q].unp);
+	    rbuf.gap5 = (int) (now.gap - fstque[q].gap);
 	}
 	fstque[q] = now;
-	if (++q == alprm2.jneibr) q = 0;
+	if (++q == qsize) q = 0;
 }
