@@ -6,7 +6,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -16,7 +16,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include <new>
@@ -75,10 +76,10 @@ CHAR complcod[] = {___,_,T,G,K,C,Y,S,B,A,W,R,D,M,H,V,N};
 CHAR aa2nuc[] = {___,_,N,C,G,A,A,G,A,A,G,A,T,T,A,T,T,C,C,C,G,A,T,G,G,A};
 
 /* max_code, amb_code, base_code, ceil_code, gap_prof, encode, decode, redctab */
-static	SEQ_CODE str_code = {28, 1, 2, 28, 28, 0, alphab, 0};
-static	SEQ_CODE nts_code = {NSIMD, N, A, N, NSIMD, nccode, nucl, ncredctab};
-static	SEQ_CODE trc_code = {TSIMD, AMB, ALA, TSIMD, TSIMD, trccode, acodon, 0};
-static	SEQ_CODE aas_code = {ASIMD, AMB, ALA, ASIMD, ASIMD, aacode, amino, aaredctab};
+static	SEQ_CODE str_code = {28, 1, 2, 28, 0, alphab, 0};
+static	SEQ_CODE nts_code = {NSIMD, N, A, N, nccode, nucl, ncredctab};
+static	SEQ_CODE trc_code = {TSIMD, AMB, ALA, TSIMD, trccode, acodon, 0};
+static	SEQ_CODE aas_code = {ASIMD, AMB, ALA, ASIMD, aacode, amino, aaredctab};
 static	const	INEX	def_inex = {0};
 
 void setdfn(const char* newdfn) {topath(seqdfn, newdfn);}
@@ -149,14 +150,13 @@ SEQ_CODE* setSeqCode(Seq* sd, const int& molc)
 
 /* examine and get the first query sequence */
 
-SeqServer::SeqServer(int ac, const char** av, InputMode infm, 
-	const char* catalog, int mq, int mt)
+SeqServer::SeqServer(int ac, const char** av, const InputMode& infm, 
+	const char* catalog, const int& mq, const int& mt)
 	: argc(ac), argc0(ac), argv(av), argv0(av), 
-	  cfrom(0), cto(0), input_form(infm)
+	  molc{mq, mt == TRON? DNA: mt},
+	  input_form(infm),
+	  input_ns(infm == IM_SNGL? 1: 2)
 {
-	molc[0] = mq;
-	molc[1] = mt == TRON? DNA: mt;
-	input_ns = (infm == IM_SNGL)? 1: 2;
 	if (algmode.blk) {
 	    target_dbf = dbs_dt[0];
 	    query_dbf = dbs_dt[1];
@@ -172,11 +172,6 @@ SeqServer::SeqServer(int ac, const char** av, InputMode infm,
 	vclear(gzfd, 2);
 #endif
 	fc = catalog? fopen(catalog, "r"): 0;
-	nfrom = counter = 0;
-	nto = INT_MAX;
-	attr[0] = attr[1] = 0;
-	atsz[0] = atsz[1] = 0;
-	sw = false;
 }
 
 void SeqServer::reset()
@@ -190,11 +185,13 @@ void SeqServer::reset()
 	if (gzfd[1]) {fclose(gzfd[1]); gzfd[1] = 0;}
 #endif
 	if (fc) rewind(fc);
-	nfrom = counter = 0;
-	nto = INT_MAX;
-	delete[] cfrom; cfrom = 0;
-	delete[] cto; cto = 0;
-	sw = false;
+	nfrom[0] = nfrom[1] = counter[0] = counter[1] = 0;
+	nto[0] = nto[1] = INT_MAX;
+	delete[] cfrom[0]; cfrom[0] = 0;
+	delete[] cfrom[1]; cfrom[1] = 0;
+	delete[] cto[0]; cto[0] = 0;
+	delete[] cto[1]; cto[1] = 0;
+	sw[0] = sw[1] = false;
 }
 
 InSt SeqServer::nextseq(Seq* sd, int which)
@@ -241,8 +238,8 @@ InSt SeqServer::nextseq(Seq* sd, int which)
 		int	c = 0;
 		while ((c = *ps) && !isspace(c) && !strchr(",-)", c)) ++ps;
 		if (c) *ps++ = '\0';
-		if (isdigit(*qs))	nfrom = atoi(qs) - 1;
-		else if (isalpha(*qs))	cfrom = strrealloc(0, qs);
+		if (isdigit(*qs))	nfrom[which] = atoi(qs) - 1;
+		else if (isalpha(*qs))	cfrom[which] = strrealloc(0, qs);
 
 		while (*ps && isspace(*ps)) ++ps;
 
@@ -250,22 +247,22 @@ InSt SeqServer::nextseq(Seq* sd, int which)
 		qs = ps;
 		while (*ps && !isspace(*ps) && *ps != ')') ++ps;
 		*ps = '\0';
-		if (isdigit(*qs))	nto = atoi(qs);
-		else if (isalpha(*qs))	cto = strrealloc(0, qs);
+		if (isdigit(*qs))	nto[which] = atoi(qs);
+		else if (isalpha(*qs))	cto[which] = strrealloc(0, qs);
 		else if (c != ',' && c != '-') {
-		    if (cfrom) cto = strrealloc(0, "");
-		    else nto = nfrom + 1;
+		    if (cfrom[which]) cto[which] = strrealloc(0, "");
+		    else nto[which] = nfrom[which] + 1;
 		}
 	    } else if ((ps = strchr(str, '$'))) {
 		char* qs = ++ps;
 		while (*ps && !isspace(*ps)) ++ps;
 		*ps = '\0';
 		if (isdigit(*qs)) {
-		    nfrom = atoi(qs);
-		    nto = nfrom + 1;
+		    nfrom[which] = atoi(qs);
+		    nto[which] = nfrom[which] + 1;
 		} else if (isalpha(*qs)) {
-		    cfrom = strrealloc(0, qs);
-		    cto = strrealloc(0, "");
+		    cfrom[which] = strrealloc(0, qs);
+		    cto[which] = strrealloc(0, "");
 		}
 	    }
 #if USE_ZLIB
@@ -282,7 +279,9 @@ InSt SeqServer::nextseq(Seq* sd, int which)
 		atsz[which] = attrsize;
 	    }
 	    strcpy(attr[which], cdr(fn));
+#if !FVAL
 	    strcat(attr[which], " S");
+#endif
 	  }
 
 // read from mulitple seq. files
@@ -292,16 +291,18 @@ InSt SeqServer::nextseq(Seq* sd, int which)
 		molc[which] = sd->inex.molc;
 		strcat(attr[which], molc[which] == PROTEIN? "P": "D");
 	    }
-	    if (sw && cto && (!*cto || (sd->sname && !wordcmp(cto, (*sd->sname)[0])))) {
-		sw = false; delete[] cto; cto = 0;
+	    if (sw[which] && cto[which] && (!*cto[which] || 
+		(sd->sname && !wordcmp(cto[which], (*sd->sname)[0])))) {
+		sw[which] = false; delete[] cto[which]; cto[which] = 0;
 	    }
-	    if (cfrom && sd->sname && !wordcmp(cfrom, (*sd->sname)[0])) {
-		sw = true; delete[] cfrom; cfrom = 0;
+	    if (cfrom[which] && sd->sname && 
+		!wordcmp(cfrom[which], (*sd->sname)[0])) {
+		sw[which] = true; delete[] cfrom[which]; cfrom[which] = 0;
 	    }
-	    if (!cfrom && nfrom == counter) sw = true;
-	    ++counter;
-	    if (sw && counter <= nto) return IS_OK;
-	    if (sw && counter > nto) return IS_END;
+	    if (!cfrom[which] && nfrom[which] == counter[which]) sw[which] = true;
+	    ++counter[which];
+	    if (sw[which] && counter[which] <= nto[which]) return IS_OK;
+	    if (sw[which] && counter[which] > nto[which]) return IS_END;
 	    else	continue;
 	   }
 	   fclose(fd[which]);
@@ -314,25 +315,27 @@ InSt SeqServer::nextseq(Seq* sd, int which)
 		molc[which] = sd->inex.molc;
 		strcat(attr[which], molc[which] == PROTEIN? "P": "D");
 	    }
-	    if (sw && cto && (!*cto || (sd->sname && !wordcmp(cto, (*sd->sname)[0])))) {
-		sw = false; delete[] cto; cto = 0;
+	    if (sw[which] && cto[which] && (!*cto[which] || 
+		(sd->sname && !wordcmp(cto[which], (*sd->sname)[0])))) {
+		sw[which] = false; delete[] cto[which]; cto[which] = 0;
 	    }
-	    if (cfrom && sd->sname && !wordcmp(cfrom, (*sd->sname)[0])) {
-		sw = true; delete[] cfrom; cfrom = 0;
+	    if (cfrom[which] && sd->sname && 
+		!wordcmp(cfrom[which], (*sd->sname)[0])) {
+		sw[which] = true; delete[] cfrom[which]; cfrom[which] = 0;
 	    }
-	    if (!cfrom && nfrom == counter) sw = true;
-	    ++counter;
-	    if (sw && counter <= nto) return IS_OK;
-	    if (sw && counter > nto) return IS_END;
+	    if (!cfrom[which] && nfrom[which] == counter[which]) sw[which] = true;
+	    ++counter[which];
+	    if (sw[which] && counter[which] <= nto[which]) return IS_OK;
+	    if (sw[which] && counter[which] > nto[which]) return IS_END;
 	    else	continue;
 	   }
 	   fclose(gzfd[which]);
 	   gzfd[which] = 0;
 	  }
 #endif
-	  delete[] cfrom; cfrom = 0;
-	  delete[] cto; cto = 0;
-	  nfrom = 0; nto = INT_MAX;
+	  delete[] cfrom[which]; cfrom[which] = 0;
+	  delete[] cto[which]; cto[which] = 0;
+	  nfrom[which] = 0; nto[which] = INT_MAX;
 	}
 	return IS_END;
 }
@@ -358,13 +361,15 @@ void Seq::refresh(const int& num, const int& length)
 	if (!is_eldest()) {
 	    sid = vrtl;
 	    seq_ = end_ = 0; cmps = 0; lens = 0; nbr = 0;
-	    sname = 0; spath = 0; exons = 0; msaname = 0; descr = 0;
+	    sname = 0; spath = 0; exons = 0; msaname = 0;
+	    exin = 0; descr = 0;
 #if USE_WEIGHT
 	    if (inex.vtwt) delete[] weight;
 	    weight = pairwt = 0;
 #endif
 	} else {
-	    delete sigII;
+	    delete sigII; 
+	    delete exin; exin = 0;
 #if USE_WEIGHT
 	    delete[] weight; weight = 0;
 	    delete[] pairwt; pairwt = 0;
@@ -383,11 +388,10 @@ void Seq::refresh(const int& num, const int& length)
 		delete descr; descr = 0;
 	    }
 	}
-	delete[] jxt; jxt = 0;
-	delete exin; exin = 0; sigII = 0; 
-	left = right = base_ = 0;
+	left = right = len = base_ = 0;
 	jscr = 0;  vrtl = 0; anti_ = 0;
-	inex = def_inex;
+	inex = def_inex; sigII = 0; 
+	delete[] jxt; jxt = 0; CdsNo = 0;
 	if (num) {
 	    seqalloc(num, length);
 	    if (!lens) lens = new int[num];
@@ -397,7 +401,7 @@ void Seq::refresh(const int& num, const int& length)
 	    if (sname)	sname->reset(num);
 	    else	sname = new Strlist(num);
 	} else {
-	    many = len = 0;
+	    many = 0;
 	}
 }
 
@@ -408,8 +412,8 @@ Seq* Seq::aliaseq(Seq* dest, const bool& this_is_alias)
 	if (!dest)	dest = new Seq(0);
 	else	dest->refresh();
 	int	destid = dest->sid;
-	memcpy(dest, this, sizeof(Seq));
-	dest->jxt = 0;
+	memcpy((void*) dest, (void*) this, sizeof(Seq));
+	dest->jxt = 0; dest->CdsNo = 0;
 	if (is_eldest()) --vrtl;
 	if (this_is_alias) {
 	    vrtl = sid;
@@ -497,9 +501,9 @@ void swapseq(Seq** x, Seq** y)
 	swap(*x, *y);
  }
 
-void Seq::seqalloc(const int& num, const int& len, const bool& keep)
+void Seq::seqalloc(const int& num, int length, const bool& keep)
 {
-	int length = len? len: DEFSEQLEN;
+	if (!length) length = DEFSEQLEN;
 	int	area = num * (length + 2);
 	if (seq_ && area <= area_) {
 	    if (num != many) {
@@ -510,15 +514,19 @@ void Seq::seqalloc(const int& num, const int& len, const bool& keep)
 	    return;
 	}
 
+	if (seq_ && !keep) {
+	    seq_ -= many;
+	    delete[] seq_;
+	}
 	CHAR*	ss = 0;
 	try {
 	    ss = new CHAR[area];
-	} catch (std::bad_alloc ba) {
+	} catch (std::bad_alloc& ba) {
 	    fatal(NoSeqSpace);
 	}
-	if (seq_) {
+	if (seq_ && keep) {
 	    seq_ -= many;
-	    if (keep) memcpy(ss, seq_, area_);
+	    memcpy(ss, seq_, area_);
 	    delete[] seq_;
 	}
 	many = byte = num;
@@ -535,11 +543,11 @@ CHAR* Seq::seq_realloc()
 
 	try {
 	    ss = new CHAR[area_];
-	} catch (std::bad_alloc ba) {
+	} catch (std::bad_alloc& ba) {
 	    fatal(NoSeqSpace);
 	}
 	memcpy(ss, seq_ -= many, area);
-	delete[] seq_ ;
+	delete[] seq_;
 	int	sz = end_ - seq_;
 	seq_ = ss + many;
 	end_ = ss + area_ - many;
@@ -610,6 +618,15 @@ Seq::Seq(Seq* sd, const int* which, const int& snl)
 	else	sd->copyseq(this, snl);
 }
 
+Seq::Seq(const Seq* sd, const bool& alias)
+{
+	memcpy((void*) this, (void*) sd, sizeof(Seq));
+	vrtl = alias;
+#if USE_WEIGHT
+	inex.vtwt = !weight;
+#endif
+}
+
 const char* Seq::path2fn(const char* pname) const
 {
 	const char*	ns = strrchr(pname, PATHDELM);
@@ -632,15 +649,29 @@ FILE* Seq::openseq(const char* str)
 	if (is_gz(pname)) {
 #if USE_ZLIB
 	    *gzfd = gzopen(pname, "r");
-	    if (*gzfd) spath = strrealloc(spath, pname);
-	    return (0);
+	    if (*gzfd) {
+		spath = strrealloc(spath, pname);
+		return (0);
+	    }
 #else
 	    fatal(gz_unsupport, pname);
 #endif
-	} 
+	}
 	FILE*	fd = fopen(pname, "r");
-	if (fd) spath = strrealloc(spath, pname);
-	return (fd);
+	if (fd) {
+	    spath = strrealloc(spath, pname);
+	    return (fd);
+	}
+#if USE_ZLIB
+	strcpy(qname, pname);
+	strcat(qname, ".gz");
+	*gzfd = gzopen(qname, "r");
+	if (*gzfd) {
+	    spath = strrealloc(spath, qname);
+	    return (0);
+	}
+#endif
+	return (0);
 }
 
 void Seq::comple()
@@ -867,12 +898,12 @@ Seq* Seq::attrseq(const char* pa)
 
 int Seq::isAmb(const CHAR& r) const
 {
-static	CHAR nucamb[] = {0,0,0,0,2,0,2,2,3,0,2,2,3,2,3,3,4};
-static	CHAR aasamb[] = {0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2};
+static	const CHAR nucamb[] = {0,0,0,0,2,0,2,2,3,0,2,2,3,2,3,3,4};
+static	const CHAR aasamb[] = {0,0,20,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2};
 
 	switch (inex.molc) {
-	    case PROTEIN: return aasamb[r];;
-	    case DNA: case RNA:	case GENOME: return (nucamb[r]);
+	    case 1: case 4: return aasamb[r];
+	    case 2: case 3: case 5: return (nucamb[r]);
 	    default:	return (0);
 	}
 }
@@ -930,8 +961,13 @@ void Seq::test_gap_amb()
 	CHAR*	tt = at(right);
 
 	inex.dels = inex.ambs = 0;
-	for ( ; ss < tt; ++ss) {
-	    if (IsGap(*ss)) inex.dels = 1;
+	int	ndel = 0;
+	for (int i = 0; ss < tt; ++ss) {
+	    if (IsGap(*ss)) ++ndel;
+	    if (++i == many) {
+		if (ndel && ndel < many) inex.dels = 1;
+		i = ndel = 0;
+	    }
 	    int	namb = isAmb(*ss);
 	    if (!namb) continue;
 	    if (isprotein()) {
@@ -1168,7 +1204,7 @@ const	CHAR*	sseq = at(left);
 	    const int*	wk = which;
 	    if (dest->sname) dest->sname->reset(dmany);
 	    else dest->sname = new Strlist(dmany);
-	    for (int i = 0; *wk >= 0; ++i, ++wk)
+	    for ( ; *wk >= 0; ++wk)
 		dest->sname->push((*sname)[*wk]);
 	}
 	if (snl & CPY_SBI) {
@@ -1177,14 +1213,11 @@ const	CHAR*	sseq = at(left);
 	}
 #if USE_WEIGHT
 // weights are normalized to sum up to many
-	if (weight) {
+	if (dest->many > 1 && weight) {
 	    if (!dest->weight) dest->weight = new FTYPE[dmany];
-	    if (dmany == 1) dest->weight[0] = 1;
-	    else { 
-		const int*	wk = which;
-		for (int i = 0; *wk >= 0; ++i, ++wk) {
-		    dest->weight[i] = weight[*wk] / nfact;
-		}
+	    const int*	wk = which;
+	    for (int i = 0; *wk >= 0; ++i, ++wk) {
+		dest->weight[i] = weight[*wk] / nfact;
 	    }
 	}
 #endif
@@ -1497,7 +1530,7 @@ char* Seq::sqline(int i, char* ps)
 	    if (pl && *pl) {
 		sname->push(pl);
 	    } else {
-		sprintf(str, "m%d", i + 1);
+		snprintf(str, 16, "m%d", i + 1);
 		sname->push(str);
 	    }
 	}
@@ -1640,7 +1673,7 @@ Seq* Seq::getseq(const char* str, DbsDt* dbf)
 		return getdbseq(dbf, str);
 	    if (!strcmp(str, "-")) {
 		fd = stdin;
-		sprintf(input, "std%d", sid);
+		snprintf(input, MAXL, "std%d", sid);
 		spath = strrealloc(spath, input);
 		if (sname) sname->assign(input);
 		else	sname = new Strlist(&input[0], "");
@@ -1959,11 +1992,11 @@ GAPS* Seq::elim_column(const int& which, float frac)
 		if (isGap(*ss)) {
 		    ++ndel;
 		    if (cond == DEL_GAP) ++k; else
-		    if (cond == DEL_GRC) w += *wt;
+		    if (cond == DEL_GRC && wt) w += *wt;
 		} else if (isAmb(*ss)) {
 		    ++namb;
 		    if (cond == DEL_AMB) ++k; else
-		    if (cond == DEL_ARC) w += *wt;
+		    if (cond == DEL_ARC && wt) w += *wt;
 		}
 		*dd++ = *ss++;
 		if (wt) ++wt;
@@ -2035,7 +2068,7 @@ Seq* Seq::randseq(double* pcmp)
 
 	refresh(1, len);
 	inex.molc = molc;
-	sprintf(name, "RAND%d", sid);
+	snprintf(name, MAXL, "RAND%d", sid);
 	if (sname) sname->assign(name);
 	else	sname = new Strlist(&name[0], "");
 	CHAR*	s = at(0);
