@@ -6,7 +6,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -16,7 +16,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #ifndef  _BSEQ_H_
@@ -26,7 +27,8 @@
 
 struct	ALPRM {float u, v, u0, u1, v0, tgapf, thr, scale, maxsp, gamma; 
 	    int k1, ls, sh, ubh, mtx_no;};
-struct	ALPRM2 {float x, y, z, o, w, bti, spb, Z, sss; int jneibr, termk1;};
+struct	ALPRM2 {float x, y, z, o, w, bti, spb, Z, sss; 
+	    int jneibr, termk1, desert;};
 struct	ALPRM3 {float scnd, hydr, hpmt; int hpwing, no_angle;};
 struct	DefSetup {int defmolc, delamb; InputMode def_input_mode;};
 
@@ -138,7 +140,6 @@ struct	SEQ_CODE {
 	int	amb_code;	// ambigity code
 	int	base_code;	// elementary code
 	int	ceil_code;	// upper bound of
-	int	gap_prof;	// deletion profile
 	CHAR*	encode;
 	char*	decode;
 	CHAR*	redctab;
@@ -152,7 +153,6 @@ struct INEX {
 	INT	cmpc:	1;
 	INT	dels:	1;
 	INT	ambs:	2;
-	INT	gfrq:	1;
 	INT	prof:	1;
 	INT	sngl:	1;
 	INT	trcv:	1;
@@ -230,7 +230,7 @@ protected:
 	CHAR*	seq_;		// sequence body
 	CHAR*	end_;		// end of seq_ memory
 	Seq**	anti_;		// rlocation of everse-complement of this
-	StrHash<int>*	mnhash;
+	StrHash<int>*	mnhash = 0;
 	void	fillpad();
 	RANGE*	setrange(const char* pa, int* ncr = 0) const;
 template <typename file_t>
@@ -258,20 +258,20 @@ mutable	int	right;		// right boundary to be operated
 	VTYPE	jscr;		// sum of weight2s
 	int*	lens;		// seq length of each member
 	int*	nbr;		// position of the first residue
-	SEQ_CODE*	code;	// code table
-	char*	spath;		// path name read from
-	char*	msaname;	// msa name
-	Strlist*	sname;	// sequence name
-	Strlist*	descr;	// descrption
-	FTYPE*	cmps;		// composition
-	JUXT*	jxt;		// HSPs
-	RANGE*	exons;		// CDS/exons in a gene seq
-	Exinon*	exin;		// statistical scores related to gene organization
-	SigII*	sigII;		// eson-intron junction signal
-	INEX	inex;		// internally used flags
+	SEQ_CODE* code = 0;	// code table
+	char*	spath = 0;	// path name read from
+	char*	msaname = 0;	// msa name
+	Strlist* sname = 0;	// sequence name
+	Strlist* descr = 0;	// descrption
+	FTYPE*	cmps = 0;	// composition
+	JUXT*	jxt = 0;	// HSPs
+	RANGE*	exons = 0;	// CDS/exons in a gene seq
+	Exinon*	exin = 0;	// statistical scores related to gene organization
+	SigII*	sigII = 0;	// eson-intron junction signal
+mutable	INEX	inex;		// internally used flags
 #if USE_WEIGHT
-	FTYPE*	weight;		// relative weight to each member
-	FTYPE*	pairwt;		// weight to each pair of members
+	FTYPE*	weight = 0;	// relative weight to each member
+	FTYPE*	pairwt = 0;	// weight to each pair of members
 	void	fpweight(FILE* fd = 0) const;	// print weight
 	FTYPE*	saveseqwt() const;	// save weight vector
 	void	restseqwt(const FTYPE* tmpwt);// retore weight vector
@@ -292,7 +292,7 @@ mutable	int	right;		// right boundary to be operated
 	bool	empty()	const {return left == right;}
 	INT	r2s(const INT& r) const {return (isprotein()? r + ALA: ((1 << r) + _));}
 	void	setmolc(const int& molc) {inex.molc = molc;}
-const	char	Strand() const {return inex.sens? '-': '+';}
+	char	Strand() const {return inex.sens? '-': '+';}
 const	char*	path2fn(const char* pname) const;
 const	char*	sqname(bool fpri = false) const {
 		if (!fpri) fpri = many > 1;
@@ -320,7 +320,7 @@ const	char*	sqname(bool fpri = false) const {
 		{return (inex.sens & REVERS)? lens[m] - n + base_: n - 1 - base_;}
 	int	senschar() const {return SensChar[inex.sens];}
 	int	index(const CHAR* ss) const {return (ss - seq_) / many;}
-	void	seqalloc(const int& num, const int& len, const bool& keep = false);
+	void	seqalloc(const int& num, int length, const bool& keep = false);
 	void	fullrange() {left = 0; right = len;}
 	void	saverange(RANGE* rng) const {rng->left = left; rng->right = right;}
 	void	restrange(const RANGE* rng) const {left = rng->left; right = rng->right;}
@@ -403,7 +403,7 @@ template <typename file_t>
 	void	pos2num(int which, int* array) const;
 	void 	putSigII(FILE* fd) const;
 	bool	findGate(RANGE* gate);
-	void	typeseq(FILE* fd = 0, bool in_line = false);
+	void	typeseq(FILE* fd = 0, const bool& in_line = false);
 	ORF*	getorf() const;
 	void	passcom(FILE* fo) const;
 	Seq*	translate(Seq* aas, ORF& orf) const;
@@ -427,6 +427,7 @@ template <typename file_t>
 	Seq(const char* fname);
 	Seq(Seq& sd, const int* which = 0, const int& snl = CPY_ALL);
 	Seq(Seq* sd, const int* which = 0, const int& snl = CPY_ALL);
+	Seq(const Seq* sd, const bool& alias);
 				// copy or extract
 	~Seq();
 friend	void 	antiseq(Seq** seqs);
@@ -435,7 +436,7 @@ friend	void 	reportseq(Seq** sqs, int n);
 };
 
 template <typename file_t>
-SeqDb* seq_NandL(int& num, int& len, int& mode, char* str, file_t fd, int molc)
+SeqDb* seq_NandL(int& num, int& leng, int& mode, char* str, file_t fd, int molc)
 {
 // Force to single sequence
 
@@ -446,7 +447,7 @@ SeqDb* seq_NandL(int& num, int& len, int& mode, char* str, file_t fd, int molc)
 // Givien Number & Length
 	if ((num = atoi(str))) {
 	    char*       ps = cdr(str);
-	    if (ps && isdigit(*ps)) len = atoi(ps);	     // Phylip like
+	    if (ps && isdigit(*ps)) leng = atoi(ps);	     // Phylip like
 	    else if (ps && *ps) fatal("Unsupported format:\n%s\n", str);
 	    if (num > 1) mode = SEQUENTIAL_MF;
 	    else	mode = SINGLE_SQ;
@@ -460,7 +461,7 @@ SeqDb* seq_NandL(int& num, int& len, int& mode, char* str, file_t fd, int molc)
 	    num = 1; mode = SINGLE_SQ;
 	    Strlist     stl(str, stddelim);
 	    int k = stl.size();
-	    len = (stl[k - 2][0] == 'N' || stl[k - 2][0] == 'Q')?
+	    leng = (stl[k - 2][0] == 'N' || stl[k - 2][0] == 'Q')?
 		atoi(stl[k - 4]): 0;
 	    return (0);
 	}
@@ -468,8 +469,8 @@ SeqDb* seq_NandL(int& num, int& len, int& mode, char* str, file_t fd, int molc)
 	for (const char* ps = str; (ps = strchr(ps, '[')); ) {
 	    num += atoi(++ps);
 	    const char* qs = strchr(ps, ':');
-	    int tl = qs? len += atoi(ps = ++qs): 0;
-	    if (tl > len) len = tl;
+	    int tl = qs? leng += atoi(ps = ++qs): 0;
+	    if (tl > leng) leng = tl;
 	}
 	if (num > 1) {mode = NATIVE_MF; return (0);}
 	SeqDb*  dbf = whichdb(str, fd);
@@ -614,8 +615,8 @@ static	const	char* readthru = "CDS may be read through !\n";
 		break;
 	    }
 	    ps = strcpy(str, ps);
-	    int	len = strlen(ps);
-	    if (!fgets(str + len, MAXL - len, fd)) {
+	    int	leng = strlen(ps);
+	    if (!fgets(str + leng, MAXL - leng, fd)) {
 		prompt(readthru);
 		break;
 	    }
@@ -778,13 +779,14 @@ template <typename file_t>
 CHAR* Seq::get_msf_aln(file_t fd, char* str, RANGE* pcr)
 {
 	int	num  = 0;
+	int	leng = 0;
 	if (!sname) sname = new Strlist;
 	while (fgets(str, MAXL, fd)) {
 	    if (!wordcmp(str, "//")) break;
 	    char*	qs = str;
 	    char*	ps = car(qs);
 	    if (!strcmp(ps, "MSF:")) {
-		len = atoi(++qs);
+		leng = atoi(++qs);
 		ps = cdr(qs); ps = cdr(ps);
 		setSeqCode(this, *ps == 'P'? PROTEIN: DNA);
 	    } else if (!strcmp(ps, "Name:")) {
@@ -795,7 +797,7 @@ CHAR* Seq::get_msf_aln(file_t fd, char* str, RANGE* pcr)
 	    }
 	}
 
-	seqalloc(num, len);
+	seqalloc(num, leng);
 	delete[]	lens;
 	lens = new int[many];
 	vclear(lens, many);
@@ -853,12 +855,15 @@ CHAR* Seq::seq_readin(file_t fd, char* str, const int& mem, RANGE* pcr, Mfile* p
 	    if (*sp == '\n') *sp = 0;
 	    else	*sp = '\0';
 	    if (algmode.mns == 0) setstrand(0, ss);
-	    while (char* sb = strchr(++ss, '|')) {
-		if (sb[1] == '\0' || isspace(sb[1])) {
-		    *sb = ' ';
-		    break;
+	    if (OutPrm.full_name) ++ss;
+	    else {
+		while (char* sb = strchr(++ss, '|')) {
+		    if (sb[1] == '\0' || isspace(sb[1])) {
+			*sb = ' ';
+			break;
+		    }
+		    ss = sb;
 		}
-		ss = sb;
 	    }
 	    sname->push(ss);
 	    if (sp) {				// process comments in header
@@ -1160,30 +1165,30 @@ const	char**	argv0;
 	gzFile	gzfd[2];
 #endif
 	FILE*	fc;
-	int	nfrom;
-	int	nto;
-	char*	cfrom;
-	char*	cto;
-	int	counter;
-	bool	sw;
+	int	nfrom[2] = {0, 0};
+	int	nto[2] = {INT_MAX, INT_MAX};
+	char*	cfrom[2] = {0, 0};
+	char*	cto[2] = {0, 0};
+	int	counter[2] = {0, 0};
+	bool	sw[2] = {false, false};
 	int	molc[2];
-	char*	attr[2];
-	int	atsz[2];
+	char*	attr[2] = {0, 0};
+	int	atsz[2] = {0, 0};
 public:
 	InputMode	input_form;
 	int	input_ns;
 	DbsDt*	target_dbf;
 	DbsDt*	query_dbf;
-	SeqServer(int ac, const char** av, InputMode infm, 
-	    const char* catalog = 0, int mq = UNKNOWN, int mt = UNKNOWN);
+	SeqServer(int ac, const char** av, const InputMode& infm, 
+	    const char* catalog = 0, const int& mq = UNKNOWN, 
+	    const int& mt = UNKNOWN);
 	~SeqServer() {
 	    if (fd[0]) fclose(fd[0]);
 	    if (fd[1]) fclose(fd[1]);
 	    if (fc) fclose(fc);
-	    delete[] attr[0];
-	    delete[] attr[1];
-	    delete[] cfrom;
-	    delete[] cto;
+	    delete[] attr[0]; delete[] attr[1];
+	    delete[] cfrom[0]; delete[] cfrom[1];
+	    delete[] cto[0]; delete[] cto[1];
 	}
 	InSt nextseq(Seq* sd, int which = 0);
 	void	reset();
@@ -1254,7 +1259,7 @@ public:
 	int	many;
 	int	pos;
 	CHAR*	res;
-	EXIN*	bb;
+	SGPT6*	bb;
 	SeqItr& operator++()
 	{
 	    ++pos;
@@ -1316,7 +1321,7 @@ public:
 	SeqItr&	reset(int n = 0, Seq* sd = 0) {
 	    if (sd) {
 		many = sd->many; res = sd->at(n);
-		bb = sd->exin? sd->exin->score(n): 0;
+		bb = sd->exin? sd->exin->score_p(n): 0;
 	    } else if (res) {
 		int shft = n - pos;
 		res += shft * many;
