@@ -9,7 +9,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -19,7 +19,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include "aln.h"
@@ -43,16 +44,17 @@ static	const	char*	Badpam = "Illegal pam = %d !\n";
 static	const	char*	strscale = "scale";
 
 //		   u,      v,   u0, u1, v0, tgapf, thr, sclase, maxsp, gamma, k1, ls, sh, ubh, mtx_no
-ALPRM	alprm = {FQUERY, FQUERY, 0., 0.6, 0, 1.0, 35., 1., 8., 0.5, 7, 1, 100, 2, 0};
-//		   x,   y,      z,   o, m, bti, spb, Z, sss, jneibr termk1
-ALPRM2	alprm2 = {30., FQUERY, FQUERY, 30., 9., 8., 20., 0., -1., 10, 45};
+ALPRM	alprm = {FQUERY, FQUERY, 0., 0.6, 0, 1.0, 35., 1., 8., 0.5, 7, 1, 100, 0, 0};
+//ALPRM	alprm = {FQUERY, FQUERY, 0., 0.6, 0, 1.0, 35., 1., 8., 0.5, 7, 1, 100, 2, 0};
+//		   x,   y,      z,   o, m, bti, spb, Z, sss, jneibr termk1 desert
+ALPRM2	alprm2 = {30., FQUERY, FQUERY, 30., 9., 8., 20., 0., -1., 10, 45, 150};
 //		scnd, hydr, hpmt, hpwing, no_angle
 ALPRM3	alprm3 = {0., 0., 0., 3, 0};
 
 BPPRM	bpprm = {0., 1., 1., 100};
 
 static	int	glocal = GLOBAL;
-static	float	smn[] = {2., 1., 0., -1., -2.};
+static	float	smn[] = {2, 1., 0., -1., FQUERY};
 static	DefPrm	defNprm[max_simmtxes+1] = 
 {{3., 8., -6., 0., 1}, {2., 6., -4., 0., 1}, {2., 4., -2., 0., 1}, {0}};
 static	DefPrm	defPprm[max_simmtxes+1] = 
@@ -145,7 +147,7 @@ void Simmtx::Nmtx()
 	dim = rows = NSIMD;
 	mtx = SquareMtx();
 	VTYPE	ntsunp = (VTYPE) -(alprm.scale * param->u);
-	setNpam(4, param->n);
+	setNpam(4, smn[4]);
 	for (int i = 1; i < NTS; ++i) {
 	    int	ii = i + gap_code;
 	    for (int j = 1; j < i; ++j) {
@@ -633,6 +635,7 @@ const	char*	vl = getarg(argc, argv, num, ++oc);
 	    case 'a': algmode.any = atoi(vl); break;	// boundary stringency
 	    case 'b': defPprm[k].b = atof(vl); break;	// bias added to mat elem
 	    case 'c': alprm2.jneibr = atoi(vl); break;	// 
+	    case 'd': alprm2.desert = atoi(vl); break;	// giveup alignment if nohit
 	    case 'e': alprm.u0 = atof(vl); break;	// background gep
 	    case 'f': alprm.v0 = atof(vl); break;	// background gop
 	    case 'g': alprm.gamma = atof(vl); break;	// gap parameter gamma
@@ -645,9 +648,10 @@ const	char*	vl = getarg(argc, argv, num, ++oc);
 	    case 'k': alprm.k1 = atoi(vl); break;	// flex point
 	    case 'l': setlsegs(atoi(vl)); break;	// # linear pieces
 	    case 'm': smn[0] = atof(vl); break;		// match
-	    case 'n': smn[4] = atof(vl);		// mismatch
+	    case 'n':
+		smn[4] = atof(vl);			// mismatch
 		if (smn[4] > 0) smn[4] = -smn[4];
-		defPprm[k].n = smn[4]; break;
+		break;
 	    case 'o': alprm2.o = atof(vl); break;	// premature termination
 	    case 'p': defPprm[algmode.crs].p = atoi(vl); break;	// 1st pam
 	    case 'q': defPprm[WlnPamNo].p = atoi(vl); break;	// 2nd pam
@@ -684,9 +688,12 @@ const	char*	vl = getarg(argc, argv, num, ++oc);
 		break;
 	    case 'J': alprm2.spb = atof(vl); break;	// matching intron position
 	    case 'K': alprm2.termk1 = atoi(vl); break;	// max terminal gap length without penalty
-	    case 'L': IntronPrm.llmt = atoi(vl); break;	// lower limit of intron
-	    case 'M': IntronPrm.maxl = int(ktof(vl)); break;	// maximum expected length of intron
+	    case 'L': IntronPrm.llmt = atoi(vl); 
+		      IntronPrm.hard_minl = 1; break;	// lower limit of intron
+	    case 'M': IntronPrm.maxl = int(ktof(vl)); 
+		      IntronPrm.hard_maxl = 1; break;	// maximum expected length of intron
 //	    case 'N': alprm2.nrmlipot = 1; break;	// normalize intron potential
+	    case 'Q': IntronPrm.nquant = atoi(vl); break;	// number of steps of rough ILD
 	    case 'S': alprm2.sss = atof(vl); 
 		if (alprm2.sss > 1.) alprm2.sss /= 100.;
 		break;
@@ -716,8 +723,12 @@ void setSimmtxes(ComPmt ab, bool mdm, DefPrm* dp)
 	int	odr[3] = {0, 1, 2};
 	if (algmode.crs) swap(odr[0], odr[1]);
 	DefPrm*	ddp = upto == 1? dp: dp + odr[0];
+	if (smn[4] == FQUERY) smn[4] = ddp->n;
+	else	ddp->n = smn[4];
 	if (alprm.v == FQUERY) alprm.v = ddp->v;
+	else	ddp->v = alprm.v;
 	if (alprm.u == FQUERY) alprm.u = ddp->u;
+	else	ddp->u = alprm.u;
 	Simmtx**	psm = simmtxes.storedSimmtx;
 	for (int mid = 0; mid < upto; ++mid, ++psm) {
 	    ddp = dp + odr[mid];
