@@ -10,7 +10,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -20,7 +20,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include "sortgrcd.h"
@@ -67,11 +68,12 @@ static	bool	reverse = false;
 
 static void usage()
 {
-	fputs("sortgrcd version 2.2: read binary grds and erds and sort them\n", stderr);
+	fputs("sortgrcd version 2.2.2: read binary .?rds and sort them\n", stderr);
 	fputs("\tin the order of chromosomal location in each direction\n", stderr);
 	fputs("Usage: sortgrcd [options] *.grd\n", stderr);
 	fputs("Note: version 2 supports outputs from spaln 2.1.0 or later\n", stderr);
 	fputs("Note: version 2.1 supports -O3, 6, 7, and 8 options\n", stderr);
+	fputs("Note: version 2.2 supports gzipped inputs\n", stderr);
 	fputs("Options:\n", stderr);
 	fputs("\t-CN:\tMinimum % of coverage (0-100)\n", stderr);
 	fputs("\t-EN:\tReport only the best (N=1) or all (N=2) results per gene locus (1)\n", stderr);
@@ -84,16 +86,18 @@ static void usage()
 	fputs("\t-ON:\tOutput format. 0:Gff3, 3:BED, 4:Native, 5:Intron, \n", stderr);
 	fputs("\t\t\t6:cDNA, 7:translate, 8:CDS, 15:unique intron\n", stderr);
 	fputs("\t-PN:\tMinimum Overall % identity (0-100)\n", stderr);
-	fputs("\t-UN:\tMaximum total number of unpaired bases in gaps\n", stderr);
-	fputs("\t-lN:\tNumber of residues per line for -O6 or -O7 (60)\n", stderr);
-	fputs("\t-mN:\tMaximum allowed missmatches at both exon boundaries\n", stderr);
-	fputs("\t-nN:\tallow non-canonical boundary? [0: no; 1: AT-AN; 2: 1bp mismatch; 3: any]\n", stderr);
-	fputs("\t-uN:\tMaximum allowed unpaired bases in gaps at both exon boundaries\n", stderr);
-	fputs("\t-gS:\tSpecify the .grp file name\n", stderr);
 	fputs("\t-Sa:\tsort chromosomes in the alphabetical order of identifier (default)\n", stderr);
 	fputs("\t-Sb:\tsort chromosomes in the order of abundance mapped on them\n", stderr);
 	fputs("\t-Sc:\tsort chromosomes in the order of apparence in the genome db\n", stderr);
 	fputs("\t-Sr:\tsort records mapped on minus strand in the reverse order of genomic positions\n", stderr);
+	fputs("\t-UN:\tMaximum total number of unpaired bases in gaps\n", stderr);
+	fputs("\t-VN:\ttMaximum memory size used for core sort (16M)\n", stderr);
+	fputs("\t-dS:\tSpecify the directory/genome_id (inferred from input)\n", stderr);
+	fputs("\t-lN:\tNumber of residues per line for -O6 or -O7 (60)\n", stderr);
+	fputs("\t-mN:\tMaximum allowed missmatches at both exon boundaries\n", stderr);
+	fputs("\t-nN:\tallow non-canonical boundary? [0: no; 1: AT-AN; 2: 1bp mismatch; 3: any]\n", stderr);
+	fputs("\t-oS:\toutput file (stdout)\n", stderr);
+	fputs("\t-uN:\tMaximum allowed unpaired bases in gaps at both exon boundaries\n", stderr);
 	exit(1);
 }
 
@@ -309,7 +313,7 @@ void Sortgrcd::print_cds(GeneRecord* gwrk, RANGE* exon, const char* Rname)
 	int	r = gwrk->Gend;
 	int	Glen = r - l;
 
-	sprintf(str, "Dbs%d %d %d", gwrk->Cid, l, r);
+	snprintf(str, MAXL, "Dbs%d %d %d", gwrk->Cid, l, r);
 	gene.getdbseq(gdbs, str, gwrk->Cid);
 	if (gwrk->Csense) gene.comrev();
 	for (RANGE* rng = fistrng(exon); neorng(rng); ++rng) {
@@ -371,7 +375,7 @@ static	const	char*	BedFrom = "%s\t%d\t%d\t%s\t%d\t%c\t%d\t%d\t%-11s\t%d\t";
 	int	r = gwrk->Gend;
 	int&	n_exon = exon->left;
 
-	sprintf(str, "Dbs%d %d %d", gwrk->Cid, l, r);
+	snprintf(str, MAXL, "Dbs%d %d %d", gwrk->Cid, l, r);
 	gene.getdbseq(gdbs, str, gwrk->Cid);
 	if (gwrk->Csense) gene.comrev();
 	for (RANGE* rng = fistrng(exon); neorng(rng); ++rng) {
@@ -755,7 +759,7 @@ const	char*	ltype = "";
 	    if (grfn >= elocus) niso = 0;
 	    fprintf(out_fd, fmt3n, gdbs->entname(gwrk->Cid), "mRNA", 
 		gwrk->Gstart, gwrk->Gend,(int) gwrk->Gscore, rvg);
-	    sprintf(mname, "%s_%d", gdbs->entname(gwrk->Cid), 
+	    snprintf(mname, MAXL, "%s_%d", gdbs->entname(gwrk->Cid), 
 		(gwrk->Gstart + gwrk->Gend) / 2000);
 	    fprintf(out_fd, fmt3m, ++Gff3MID, Gff3GID, mname);
 	    for (INT m = 0; m < gwrk->nexn; ++m, ++ewrk) {
@@ -997,8 +1001,11 @@ const	char*	errmsg = "%s may be obolete or corrupted!\n";
 	    fclose(gzfd);
 	    return (nr);
 	}
+	char*	dot = strrchr(str, '.');
+	if (dot) *dot = '\0';
+	strcat(str, "(.gz)");
 #endif
-	fatal(errmsg, str);
+	fatal(not_found, str);
 	return (0);
 }
 
@@ -1038,7 +1045,14 @@ Sortgrcd::Sortgrcd(int ac, const char** av) : argc(ac), grdname(*av)
 	nrcd = (GERecN*) mfd.flush();
 	ngrcd = gerNo.gcn;
 	nercd = gerNo.ecn;
-	gdbs = new DbsDt((*gerNo.sname)[0]);
+	if (!gdbs) gdbs = new DbsDt((*gerNo.sname)[0]);
+	strcpy(str, gdbs->dbsid);
+	strcat(str, ".grp");
+	FILE*	fd = fopen(str, "r");
+	if (!fd) fatal("%s not found !\n", str);
+	while (fgets(str, MAXL, fd)) ;
+	fclose(fd);
+	HashSize = atoi(cdr(str));	// read from the last line
 }
 
 Sortgrcd::~Sortgrcd()
@@ -1047,10 +1061,10 @@ Sortgrcd::~Sortgrcd()
 	delete[] chrlist; delete[] grcd; delete[] nrcd;
 }
 
-void Sortgrcd::readGrcd(int ac, const char** av, INT hashsize)
+void Sortgrcd::readGrcd(int ac, const char** av)
 {
 // readn G-records
-	Chash*	hh = new Chash(hashsize);
+	Chash*	hh = new Chash(HashSize);
 	grcd = new GRFn[ngrcd];
 	GRFn*	grfn = grcd;
 	char	str[LINE_MAX];
@@ -1128,13 +1142,9 @@ static	const	int	no_cp = 10;
 	ftime(tb);
 #endif
 
-const	char*	grpfn = 0;
-	char	str[MAXL];
-	INT	HashSize = 1033;
-
 // Get options
 	while (--argc && (++argv)[0][0] == OPTCHAR) {
-	  const char*	val = argv[0] + 2;
+const	  char*	val = argv[0] + 2;
 	  int	flevel = 0;
 	  switch (argv[0][1]) {
 	    case 'F': 
@@ -1207,13 +1217,9 @@ const	char*	grpfn = 0;
 		    }
 		  }
 		  break;
-	    case 'g':
-		if ((val = getarg(argc, argv)))		// .grp file name
-		    {grpfn = val;}
-		break;
-	    case 'h':
-		if ((val = getarg(argc, argv, true)))	// hash size
-		    {HashSize = atoi(val);}
+	    case 'd':
+		if ((val = getarg(argc, argv)))		// gdbs id
+		    {gdbs = new DbsDt(val);}
 		break;
 	    case 'l':
 		if ((val = getarg(argc, argv)))		// max number of mismatches
@@ -1228,6 +1234,10 @@ const	char*	grpfn = 0;
 		    {filter.ncan = atoi(val);}		// non-canonical ends
 		else	{filter.ncan = 0;}
 		break;
+	    case 'o':
+		if ((val = getarg(argc, argv)))		// max number of mismatches
+		    {OutPrm.out_file = val;}
+		break;
 #if M_THREAD
 	    case 't':
 		if ((val = getarg(argc, argv)))		// number of thread
@@ -1244,13 +1254,7 @@ const	char*	grpfn = 0;
 	}
 	if (!argc) usage();
 	setup_output(OutMode);
-	if (grpfn) {	// get seq size from *.grp 
-	    FILE*	fd = fopen(grpfn, "r");
-	    if (!fd) fatal("%s not found !\n", grpfn);
-	    while (fgets(str, MAXL, fd)) ;
-	    fclose(fd);
-	    HashSize = atoi(cdr(str));	// read from the last line
-	}
+
 #if MONITOR
 	times(tt + cp);
 	ftime(tb + cp++);
@@ -1264,7 +1268,7 @@ const	char*	grpfn = 0;
 	ftime(tb + cp++);
 #endif
 // read and sort G-records
-	srtgrcd.readGrcd(argc, argv, HashSize);
+	srtgrcd.readGrcd(argc, argv);
 
 #if MONITOR
 	times(tt + cp);
