@@ -9,17 +9,18 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
 *
-*	Osamu Gotoh, Ph.D.      (2003-2012)
+*	Osamu Gotoh, Ph.D.      (2003-)
 *	Department of Intelligence Science and Technology
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include "aln.h"
@@ -27,7 +28,6 @@
 #include "vmf.h"
 #include "wln.h"
 #include "blksrc.h"
-#include <math.h>
 
 #ifndef	M_THREAD
 #define	M_THREAD	1
@@ -39,10 +39,11 @@
 #include <sched.h>
 #endif
 
-#define	USE_FULL_RANGE	0
 #define	TESTRAN		0
 #define	QDEBUG	0
 
+enum QvsD {UNDF, AvsA, AvsG, CvsC, CvsG, FvsG, GvsA, GvsC, GvsG};
+
 static	const	int	MAX_ARGS = 128;
 static	const	int	MAX_ARGTEXT = 4096;
 static	const	int	InsPam = 100;	// intra species
@@ -52,11 +53,6 @@ static	const	int	DefOrfLen = 75;
 static	const	int	def_max_extend_gene_rng = 3;
 static	const	long	MaxWordNoSpace = 1 << 30;	// 1GB
 
-enum QvsD {UNDF, AvsA, AvsG, CvsC, CvsG, FvsG, GvsA, GvsC, GvsG};
-
-static	QvsD	QRYvsDB = CvsG;
-static	InputMode	input_form = IM_SNGL;
-
 class HalfGene {
 public:
 	int	left;
@@ -116,7 +112,7 @@ class	ThQueue;
 #endif	// M_THREAD
 
 static	void	usage(const char* messg);
-static	int	getoption(int argc, const char** argv);
+static	int	getoption(int argc, const char** argv, const int visit = 0);
 static	void	readargs();
 static	RANGE*	skl2exrng(SKL* skl);
 static	int	spalign2(Seq* sqs[], PwdB* pwd, Gsinfo* GsI, int ori = 1);
@@ -136,6 +132,11 @@ static	const	char*	catalog = 0;
 static	const	char*	aadbs = 0;
 static	const	char*	genomedb = 0;
 static	const	char*	cdnadb = 0;
+
+static	QvsD	QRYvsDB = CvsG;
+static	InputMode	input_form = IM_SNGL;
+static	int	extra_seq = 1;
+static	int	b_intr = 0;
 static	bool	WriteMolc = false;
 static	int	QryMolc = UNKNOWN;
 static	int	TgtMolc = UNKNOWN;
@@ -144,14 +145,26 @@ static	int	g_segment = 2 * MEGA;
 static	int	q_mns = 3;
 static	int	no_seqs = 3;
 static	bool	gsquery = QRYvsDB == GvsA || QRYvsDB == GvsC;
-static	const	char*	version = "2.4.9a";
-static	const	int	date = 220512;
+static	const	char*	version = "3.0.7";
+static	const	int	date = 250522;
 static	AlnOutModes	outputs;
 
 static void usage(const char* messg)
 {
+#if __AVX512BW__
+const	char*	arch = "_AVX512";
+#elif __AVX2__
+const	char*	arch = "_AVX2";
+#elif __SSE4_1__
+const	char*	arch = "_SSE4.1";
+#elif __ARM_NEON
+const	char*	arch = "_ARM_NEON";
+#else
+const	char*	arch = "";
+#endif
 	if (messg && *messg) fputs(messg, stderr);
-	fprintf(stderr, "\n*** SPALN version %s <%d> ***\n\n", version, date);
+	fprintf(stderr, "\n*** SPALN%s version %s <%d> ***\n\n", 
+	    arch, version, date);
 	fputs("Usage:\n", stderr);
 	fputs("spaln -W[Genome.bkn] -KD [W_Options] Genome.mfa\t(to write block inf.)\n", stderr);
 	fputs("spaln -W[Genome.bkp] -KP [W_Options] Genome.mfa\t(to write block inf.)\n", stderr);
@@ -165,7 +178,6 @@ static void usage(const char* messg)
 	fputs("spaln [R_options] -aAAdb protein.fa\t(to search aa database)\n", stderr);
 	fputs("\nin the following, # = integer or real number; $ = string; default in ()\n\n", stderr);
 	fputs("W_Options:\n", stderr);
-	fputs("\t-E\tGenerate local lookup table for each block\n", stderr);
 	fputs("\t-XC#\tnumber of bit patterns < 6 (1)\n", stderr);
 	fputs("\t-XG#\tMaximum expected gene size (inferred from genome|db size)\n", stderr);
 	fputs("\t-Xk#\tWord size (inferred from genome|db size)\n", stderr);
@@ -175,7 +187,7 @@ static void usage(const char* messg)
 	fputs("\t-g\tgzipped output\n", stderr);
 	fputs("\t-t#\tMutli-thread operation with # threads\n\n", stderr);
 	fputs("R_Options (representatives):\n", stderr);
-	fputs("\t-E\tUse local lookup table for each block\n", stderr);
+	fputs("\t-A[0-3]\t0: scalar, 1..3: simd; 1: rigorous, 2: intermediate, 3: fast\n", stderr);
 	fputs("\t-H#\tMinimum score for report (35)\n", stderr);
 	fputs("\t-L or -LS or -L#\tsemi-global or local alignment (-L)\n", stderr);
 	fputs("\t-M#[,#2]\tNumber of outputs per query (1) (4 if # is omitted)\n", stderr);
@@ -198,12 +210,13 @@ static void usage(const char* messg)
 	fputs("\t-l#\tNumber of characters per line in alignment (60)\n", stderr);
 	fputs("\t-o$\tFile/directory/prefix where results are written (stdout)\n", stderr);
 	fputs("\t-pa#\tRemove 3' poly A >= # (0: don't remove)\n", stderr);
+	fputs("\t-pF\tOutput full Fasta entry name\n", stderr);
+	fputs("\t-pj\tSuppress splice junction information with -O[6|7]\n", stderr);
 	fputs("\t-pn\tRetain existing output file\n", stderr);
 	fputs("\t-po\tOverwrite existing output file\n", stderr);
 	fputs("\t-pw\tReport results even if the score is below the threshold\n", stderr);
 	fputs("\t-pT\tExclude termination codon from CDS\n", stderr);
 	fputs("\t-r$\tReport information about block data file\n", stderr);
-	fputs("\t-r$\tReport information about block data file\n", stderr);
 	fputs("\t-u#\tGap-extension penalty (3)\n", stderr);
 	fputs("\t-v#\tGap-open penalty (8)\n", stderr);
 	fputs("\t-w#\tBand width for DP matrix scan (100)\n", stderr);
@@ -219,22 +232,23 @@ static void usage(const char* messg)
 	fputs("\t-yB#\tWeight for branch point signal (0)\n", stderr);
 	fputs("\t-yI$\tIntron length distribution\n", stderr);
 	fputs("\t-yL#\tMinimum expected length of intron (30)\n", stderr);
+	fputs("\t-yM#\tMaximum length of intron (unset)\n", stderr);
 	fputs("\t-yS[#]\tUse species-specific parameter set (0.0/0.5)\n", stderr);
 	fputs("\t-yX0\tDon't use parameter set for cross-species comparison\n", stderr);
 	fputs("\t-yZ#\tWeight for intron potential (0)\n", stderr);
 	fputs("\t-XG#\tReset maximum expected gene size, suffix k or M is effective\n", stderr);
 	fputs("\nExamples:\n", stderr);
-	fputs("\tspaln -W -KP -E -t4 dictdisc_g.gf\n", stderr);
+	fputs("\tspaln -W -KP -t4 dictdisc_g.gf\n", stderr);
 	fputs("\tspaln -W -KA -Xk5 Swiss.faa\n", stderr);
-	fputs("\tspaln -O -LS \'chr1.fa 10001 40000\' cdna.nfa\n", stderr);
-	fputs("\tspaln -Q0,1,7 -t10 -TTetrapod -XG2M -ommu/ -dmus_musc_g hspcdna.nfa\n", stderr);
-	fputs("\tspaln -Q7 -O5 -t10 -Tdictdics -ddictdisc_g [-E] \'dictdisc.faa (101 200)\' > ddi.intron\n", stderr);
+	fputs("\tspaln -O1 -LS \'chr1.fa 10001 40000\' cdna.nfa\n", stderr);
+	fputs("\tspaln -Q7 -O0,1,7 -t10 -TTetrapod -XG2M -ommu/ -dmus_musc_g hspcdna.nfa\n", stderr);
+	fputs("\tspaln -Q7 -O5 -t10 -Tdictdics -ddictdisc_g\'dictdisc.faa (101 200)\' > ddi.intron\n", stderr);
 	fputs("\tspaln -Q7 -O0 -t10 -Tdictdics -aSwiss \'chr1.nfa 200001 210000\' > Chr1_200-210K.gff\n", stderr);
 	fputs("\tspaln -Q4 -O0 -t10 -M10 -aSwiss dictdisc.faa > dictdisc.alignment_score\n", stderr);
 	exit(1);
 }
 
-static int getoption(int argc, const char** argv)
+static int getoption(int argc, const char** argv, const int visit)
 {
 const	char**	argbs = argv;
 	DbsDt**	dbs = dbs_dt;
@@ -248,10 +262,23 @@ const	    char*	val = argv[0] + 2;
 	    int	rv = 0;
 	    switch (c) {
 		case '?': case 'h': usage(0);
-		case 'a': case 'A':
-		    algmode.dim = c == 'A';
-		    if ((val = getarg(argc, argv)))
-			{delete *dbs; *dbs = new DbsDt(aadbs = val);}
+		case 'a':
+		    algmode.dim = 0;
+		    if ((val = getarg(argc, argv))) {
+			delete *dbs;
+			*dbs = new DbsDt(aadbs = val);
+		    }
+		    break;
+		case 'A':
+		    if ((val = getarg(argc, argv))) {
+			if (isdigit(*val))
+			    algmode.alg = atoi(val);
+			else {
+			    algmode.dim = 1;	// on memory dbs
+			    delete *dbs;
+			    *dbs = new DbsDt(aadbs = val);
+			}
+		    }
 		    break;
 		case 'd': case 'D': 
 		    algmode.dim = c == 'D';
@@ -266,12 +293,6 @@ const	    char*	val = argv[0] + 2;
 		    if ((val = getarg(argc, argv, true)))
 			initcodon(atoi(val));
 		    break;
-		case 'E': 	// use lookup tabble
-		    if ((val = getarg(argc, argv, true)))
-			algmode.alg = atoi(val);
-		    else
-			algmode.alg |= 1;
-		    break;
 		case 'f':
 		    if ((val = getarg(argc, argv))) catalog = val;
 		    break;
@@ -351,6 +372,8 @@ const	    char*	val = argv[0] + 2;
 			    case 'U': algmode.lcl = 12; break;
 			    case 's':	// Smith-Waterman
 			    case 'S': algmode.lcl = 16; break;
+			    case 'c':	// Core 
+			    case 'C': algmode.lcl = 48; break;
 			    default:  algmode.lcl = 15; break;
 			}
 		    }
@@ -398,6 +421,7 @@ const	    char*	val = argv[0] + 2;
 			case 'D': OutPrm.debug = 1; break;
 			case 'e': OutPrm.trimend = !OutPrm.trimend; break;
 			case 'f': OutPrm.deflbl = 1; break;
+			case 'F': OutPrm.full_name = 1; break;
 			case 'i': OutPrm.ColorEij = 1; break;
 			case 'j': OutPrm.spjinf = 1 - OutPrm.spjinf; break;
 			case 'n': OutPrm.overwrite = 2; break;
@@ -406,7 +430,7 @@ const	    char*	val = argv[0] + 2;
 			case 'Q': setprompt(0, 1);  break;
 			case 'r': algmode.mlt = 1; break;
 			case 't': OutPrm.deflbl = 2; break;
-			case 'w': algmode.thr = 0; break;
+			case 'w': OutPrm.all_out = 1; break;
 			case 'x': OutPrm.supself = (val[1] == '2')? 2: 1; break;
 			case 'J': case 'm': case 'u': case 'v': 
 			    setexprm_z(argc, argv); break;
@@ -460,7 +484,6 @@ const	    char*	val = argv[0] + 2;
 		case 'T':
 		    if ((val = getarg(argc, argv)))
 			ftable.setpath(val, gnm2tab);
-		    readargs();
 		    break;
 		case 'u': case 'v': case 'w': 
 		    readalprm(argc, argv); break;
@@ -488,6 +511,7 @@ const	    char*	val = argv[0] + 2;
 	    }
 	    if (*dbs && (dbs + 1 < dbs_dt + MAX_DBS)) ++dbs;
 	}
+	if (visit == 0) readargs();
 	return (argv - argbs);
 }
 
@@ -525,13 +549,14 @@ const	char*	argv[MAX_ARGS];
 	}
 readend:
 	fclose(fd);
-	getoption(argc, argv);
+	getoption(argc, argv, 1);
 }
 
 void AlnOutModes::alnoutput(Seq** sqs, Gsinfo* GsI)
 {
 	int	print2Skip = 0;
-const	bool	swp = sqs[1]->inex.intr;
+const	bool	swp = sqs[1]->inex.intr || 
+		(sqs[0]->isprotein() && sqs[1]->istron());
 	Seq*	gene = 0;
 	GAPS*	gaps[2] = {0, 0};
 
@@ -640,16 +665,14 @@ static	RANGE*	skl2exrng(SKL* skl)
 
 static int spalign2(Seq* sqs[], PwdB* pwd, Gsinfo* GsI, int ori)
 {
-	if (algmode.lsg && pwd->DvsP != 3)
-	    genomicseq(sqs + 1, pwd, ori);
 	switch (QRYvsDB) {
 	  case AvsG: 
-	  case GvsA: GsI->skl = alignH_ng((const Seq**) sqs, pwd, &GsI->scr); break;
+	  case GvsA: GsI->skl = alignH_ng((const Seq**) sqs, pwd, GsI); break;
 	  case GvsC:
-	  case CvsG: GsI->skl = alignS_ng(sqs, pwd, &GsI->scr, ori); break;
+	  case CvsG: GsI->skl = alignS_ng(sqs, pwd, GsI, ori); break;
 	  case FvsG: 
 	  case AvsA: 
-	  case CvsC: GsI->skl = alignB_ng((const Seq**) sqs, pwd, &GsI->scr); break;
+	  case CvsC: GsI->skl = alignB_ng((const Seq**) sqs, pwd, GsI); break;
 	  default:
 		prompt("Improper combination of db and query!\n");
 		return (ERROR);
@@ -668,8 +691,6 @@ static int spalign2(Seq* sqs[], PwdB* pwd, Gsinfo* GsI, int ori)
 		GsI->CDSrng = skl2exrng(GsI->skl);
 	    }
 	} else {
-//	    prompt("%s %s failed to align!\n", 
-//		sqs[0]->sname(), sqs[1]->sname());
 	    return (ERROR);
 	}
 	return (OK);
@@ -712,26 +733,29 @@ static	PwdB* SetUpPwd(Seq* sqs[])
 
 static int match_2(Seq* sqs[], PwdB* pwd, ThQueue* q)
 {
+	if (gsquery) std::swap(sqs[0], sqs[1]);
 	Seq*&	a = sqs[0];
 	Seq*&	b = sqs[1];
-	int	dir = a->isprotein() + 2 * b->isprotein();
+	Seq*&	c = sqs[2];
 
+	INT	ori = a->inex.ori;
+	b->inex.intr = b_intr;
+	if (pwd->DvsP == 0) {
+	    if (!c->len) b->comrev(sqs + 2);
+	    if (!b->exin) b->exin = new Exinon(b, pwd, ori == 3);
+	    if (!c->exin) c->exin = new Exinon(c, pwd, ori == 3);
+	} else if (pwd->DvsP == 1) {
+	    if (!c->len) b->comrev(sqs + 2);
+	    b->nuc2tron();
+	    c->nuc2tron();
+	    if (!b->exin) b->exin = new Exinon(b, pwd, ori == 3);
+	    if (!c->exin) c->exin = new Exinon(c, pwd, ori == 3);
+	}
+
+	int	dir = a->isprotein() + 2 * b->isprotein();
 	if (dir != pwd->DvsP)
 	    fatal("Don't mix different combinations of seq. types %s %d %d\n",
 		(*a->sname)[0], dir, pwd->DvsP);
-	if (gsquery) std::swap(sqs[0], sqs[1]);
-	if (QRYvsDB == GvsA || QRYvsDB == AvsG) b->inex.intr = algmode.lsg;
-	int	ori = a->inex.ori;
-	if (pwd->DvsP != 3) {
-	    if (algmode.mns == 0 || algmode.mns == 3 || ori == 3) {
-		if (pwd->DvsP == 1) b->nuc2tron();
-		geneorient(sqs, pwd);
-	    }
-	    if (algmode.mns == 1 && ori == 3) ori = b->inex.sens? 2: 1;
-	    if (algmode.mns == 2 && ori == 3) ori = b->inex.sens? 1: 2;
-	    if (algmode.lsg == 0 && ori == 3) ori = 1;
-	    if (b->inex.intr && dbs_dt[0]) extend_gene_rng(sqs, pwd, dbs_dt[0]);
-	}
 	if (algmode.lcl & 16) {	// local
 	    a->exg_seq(1, 1);
 	    b->exg_seq(1, 1);
@@ -739,25 +763,83 @@ static int match_2(Seq* sqs[], PwdB* pwd, ThQueue* q)
 	    a->exg_seq(algmode.lcl & 4, algmode.lcl & 8);
 	    b->exg_seq(algmode.lcl & 1, algmode.lcl & 2);
 	}
-	Gsinfo	GsI[2];
-	dir = spalign2(sqs, pwd, GsI, ori);
-	if (dir != ERROR && (!algmode.thr || GsI->fstat.val >= pwd->Vthr)) {
+
+	int	nparalog = 0;
+	if (pwd->DvsP == 3) nparalog = 1;
+	else {
+	    if (algmode.mns == 0 || algmode.mns == 3 || ori == 3) {
+		if (pwd->DvsP == 1) b->nuc2tron();
+		nparalog = geneorient(sqs, pwd);
+	    }
+	    if (algmode.mns == 1 && ori == 3) ori = b->inex.sens? 2: 1;
+	    if (algmode.mns == 2 && ori == 3) ori = b->inex.sens? 1: 2;
+	    if (algmode.lsg == 0 && ori == 3) ori = 1;
+	    if (b->inex.intr && dbs_dt[0]) extend_gene_rng(sqs, pwd, dbs_dt[0]);
+	}
+	if (nparalog == 0 && (ori == 1 || ori == 2)) nparalog = 1;
+	if (nparalog == 0) {
+	    if (gsquery) std::swap(sqs[0], sqs[1]);
+	    cleanseq(sqs + 2, 2, 1);
+	    return (0);
+	}
+	Gsinfo*	GsI = new Gsinfo[nparalog + 1];
+	Gsinfo*	gsinf = GsI;
+	INT	n_out = 0;
+	Seq**	gener = sqs + 3;
+	for (int n = 0; n < nparalog; ++n, ++gsinf) {
+	    if (n) std::swap(b, gener[n - 1]);
+	    dir = spalign2(sqs, pwd, gsinf, ori);
+	    if (n) std::swap(b, gener[n - 1]);
+	    if (dir != ERROR && 
+		(OutPrm.all_out || GsI->fstat.val > pwd->Vthr)) {
+		++n_out;
+		if (algmode.nsa == BED_FORM)
+		    GsI->rscr = selfAlnScr(a, pwd->simmtx);
+	    } else  GsI->scr = NEVSEL;
+	}
+
+//	insert sort GsI as nearly sorted
+	int*	odr = 0;
+	if (nparalog > 1) {
+	    odr = new int[nparalog];
+	    for (int n = 0; n < nparalog; ++n) odr[n] = n;
+	    for (int n = 1; n < nparalog; ++n) {
+		int	l = odr[n];
+		VTYPE	v = GsI[l].fstat.val;
+		int	m = n;
+		while (--m >= 0 && v > GsI[odr[m]].fstat.val)
+		    odr[m + 1] = odr[m];
+		odr[m + 1] = l;
+	    } 
+	}
+	if (OutPrm.MaxOut < n_out) n_out = OutPrm.MaxOut;
+	for (INT n = 0; n < n_out; ++n) {
+	    int	k = odr? odr[n]: 0;
+	    if (k) std::swap(b, gener[k - 1]);
+	    Gsinfo*	gsinf = GsI + k;
+	    if (bool(gsinf->skl->m & A_RevCom) ^ bool(a->inex.sens))
+		a->comrev();
+	    if (gsquery) std::swap(sqs[0], sqs[1]);
 	    if (algmode.nsa == BED_FORM)
-		GsI->rscr = selfAlnScr(a, pwd->simmtx);
+		gsinf->rscr = selfAlnScr(a, pwd->simmtx);
 #if M_THREAD
 	    if (q) {
 		pthread_mutex_lock(&q->mutex);
-		outputs.alnoutput(sqs, GsI);
+		outputs.alnoutput(sqs, gsinf);
 		pthread_mutex_unlock(&q->mutex);
 	    } else
 #endif
-		outputs.alnoutput(sqs, GsI);
+		outputs.alnoutput(sqs, gsinf);
+	    if (gsquery) std::swap(sqs[0], sqs[1]);
+	    if (k) std::swap(b, gener[k - 1]);
 	}
 	if (dir == 1) {
 	    a->comrev();
 	    antiseq(sqs + 1);
 	}
+	delete[] GsI; delete[] odr;
 	if (gsquery) std::swap(sqs[0], sqs[1]);
+	cleanseq(sqs + 2 + extra_seq, nparalog - extra_seq, 1);
 	return (dir);
 }
 
@@ -769,7 +851,6 @@ static int blkaln(Seq* sqs[], SrchBlk* bks, RANGE* rng, ThQueue* q)
 	RANGE	frng = {INT_MAX, 0};
 	Seq*&	a = sqs[0];
 	Seq*&	b = sqs[1];
-	int	b_intr = b->inex.intr;
 
 	if (gsquery) {
 	    a->saverange(&grng);
@@ -778,6 +859,8 @@ static int blkaln(Seq* sqs[], SrchBlk* bks, RANGE* rng, ThQueue* q)
 	    frng.right = grng.right - n;
 	}
 	bks->setseqs(sqs);
+	Seq**	gener = bks->get_gener();
+const	int	basis = gener - sqs;
 
 	if (algmode.lcl & 16) a->exg_seq(1, 1);	// SWG local alignment
 	else	a->exg_seq(algmode.lcl & 4, algmode.lcl & 8);
@@ -787,17 +870,15 @@ static int blkaln(Seq* sqs[], SrchBlk* bks, RANGE* rng, ThQueue* q)
 	    nparalog = bks->findh(sqs);
 	else
 	    nparalog = bks->findblock(sqs);
-	if (nparalog == ERROR || algmode.nsa == MAP1_FORM || algmode.nsa == MAP2_FORM)
+	if (nparalog == ERROR || 
+	    algmode.nsa == MAP1_FORM || algmode.nsa == MAP2_FORM) {
+	    cleanseq(sqs + 2, basis - 1, 1);
 	    return (0);		// no alignment
-#if USE_FULL_RANGE
-// skip the next line if only the specified range is used as the query
-	if (rng) a->right = a->right == rng->right? a->len: rng->right;
-#endif
+	}
 	HalfGene	hfg(0, INT_MAX, a->CdsNo);
 	Gsinfo*	GsI = new Gsinfo[nparalog + 1];
 	Gsinfo*	gsinf = GsI;
 	INT	n_out = 0;
-	Seq**	gener = bks->get_gener();
 	for (int n = 0; n < nparalog; std::swap(b, gener[n++]), ++gsinf) {
 	    std::swap(b, gener[n]);
 	    if ((QRYvsDB == AvsA || QRYvsDB == CvsC) && OutPrm.supself) {
@@ -826,10 +907,12 @@ static int blkaln(Seq* sqs[], SrchBlk* bks, RANGE* rng, ThQueue* q)
 	    }				// incompatible gene orientation
 	    if (algmode.lcl & 16) b->exg_seq(1, 1);
 	    else	b->exg_seq(algmode.lcl & 1, algmode.lcl & 2);
+	    if (bks->pwd->DvsP != 3)
+		genomicseq(sqs + 1, bks->pwd, a->inex.ori);
 	    int	dir = spalign2(sqs, bks->pwd, gsinf, a->inex.ori);
 	    delete b->exin; b->exin = 0;	// suppress Boundary output
-	    if (dir < 0 || !gsinf->skl ||
-		(algmode.thr && gsinf->scr < bks->pwd->Vthr)) {
+	    if (dir < 0 || !gsinf->skl || 
+		(!OutPrm.all_out && gsinf->scr <= bks->pwd->Vthr)) {
 		gsinf->scr = NEVSEL;
 		continue;
 	    }
@@ -888,8 +971,9 @@ static int blkaln(Seq* sqs[], SrchBlk* bks, RANGE* rng, ThQueue* q)
 	if (OutPrm.MaxOut < n_out) n_out = OutPrm.MaxOut;
 	for (INT n = 0; n < n_out; ++n) {
 	    int	k = odr? odr[n]: 0;
-	    std::swap(b, gener[k]);
 	    Gsinfo*	gsinf = GsI + k;
+	    if (!gsinf->skl) continue;
+	    std::swap(b, gener[k]);
 	    if (bool(gsinf->skl->m & A_RevCom) ^ bool(a->inex.sens))
 		a->comrev();
 	    if (gsquery) std::swap(sqs[0], sqs[1]);
@@ -924,8 +1008,8 @@ static int blkaln(Seq* sqs[], SrchBlk* bks, RANGE* rng, ThQueue* q)
 	    pthread_mutex_unlock(&q->mutex);
 	}
 #endif
-	cleanseq(gener, OutPrm.MaxOut2 + 1, 1);
 	delete[] GsI; delete[] odr;
+	cleanseq(sqs + 2, basis + nparalog - 1, 1);
 	return (hfg.right);
 }
 
@@ -938,6 +1022,7 @@ static SrchBlk* getblkinf(Seq* sqs[], const char* dbs, MakeBlk* mb)
 	int	ap = a->isprotein();
 	int	bp = mb? mb->dbsmolc() == PROTEIN: ((dbs && (dbs == aadbs))? 1: b->isprotein());
 	char	str[LINE_MAX];
+	int	dvsp = 0;
 
 	if (genomedb) {
 	    b->inex.molc = DNA;
@@ -961,6 +1046,7 @@ static SrchBlk* getblkinf(Seq* sqs[], const char* dbs, MakeBlk* mb)
 	    b->inex.intr = algmode.lsg;
 	    a->inex.intr = 0;
 	} 
+	b_intr = b->inex.intr;
 	b->byte = b->many = 1;
 	if (ap && bp)	{QRYvsDB = AvsA; algmode.lsg = 0;}
 	else if (ap)	QRYvsDB = AvsG;
@@ -977,7 +1063,7 @@ static SrchBlk* getblkinf(Seq* sqs[], const char* dbs, MakeBlk* mb)
 	    std::swap(sqs[0], sqs[1]);
 	    std::swap(ap, bp);
 	}
-	makeWlprms(prePwd((const Seq**) sqs));
+	makeWlprms((dvsp = prePwd((const Seq**) sqs)));
 	if (!ReadBlock && dbs) {
 	    const char*	ext = 0;
 	    switch (QRYvsDB) {
@@ -997,6 +1083,7 @@ static SrchBlk* getblkinf(Seq* sqs[], const char* dbs, MakeBlk* mb)
 	if ((QRYvsDB == CvsC || QRYvsDB == FvsG) && algmode.mns != 2) algmode.mns = 1;
 	SrchBlk* bks = dbs? new SrchBlk(sqs, ReadBlock, genomedb):
 		new SrchBlk(sqs, mb, b->inex.intr);
+	bks->pwd->DvsP = dvsp;
 	if (MinQueryLen == 0) MinQueryLen = bks->MinQuery();
 	if (algmode.mlt == 1) MinSegLen = MinQueryLen;
 	if (ap) q_mns &= ~2;
@@ -1052,25 +1139,30 @@ static void genomicseq(Seq** sqs, PwdB* pwd, int ori)
 	sqs[0]->inex.intr = algmode.lsg;
 	if (ori & 1) {
 	    if (pwd->DvsP == 1) sqs[0]->nuc2tron();
-	    if (algmode.lsg) sqs[0]->exin = new Exinon(sqs[0], pwd, ori == 3);
+	    if (!sqs[0]->exin)
+		sqs[0]->exin = new Exinon(sqs[0], pwd, ori == 3);
 	}
 	if (ori & 2 && pwd->DvsP != 3) {
 	    sqs[0]->comrev(sqs + 1);
 	    sqs[1]->setanti(sqs);
 	    if (pwd->DvsP == 1) sqs[1]->nuc2tron();
-	    if (algmode.lsg) sqs[1]->exin = new Exinon(sqs[1], pwd, ori == 3);
+	    if (!sqs[1]->exin)
+		sqs[1]->exin = new Exinon(sqs[1], pwd, ori == 3);
 	}
 }
 
 static void spaln_job(Seq* sqs[], void* prm, ThQueue* q)
 {
 	Seq*&	a = sqs[0];
+	PwdB*	pwd = (PwdB*) prm;
+
 	a->inex.intr = (gsquery)? algmode.lsg: 0;
 	if (a->isdrna() && !gsquery && q_mns)	 	// cDNA
 	    a->inex.ori = polyA.rmpolyA(a, a->sigII? 1: q_mns);
 	else	a->inex.ori = q_mns? q_mns: 3;
+	if (algmode.crs) Wlp	wlp(sqs);		// mask low ic region
 	if (algmode.blk) (void) quick4(sqs, (SrchBlk*) prm, q);
-	else	match_2(sqs, (PwdB*) prm, q);
+	else	match_2(sqs, pwd, q);
 }
 
 static void seg_job(Seq** sqs, SeqServer* svr, SrchBlk* sbk)
@@ -1255,10 +1347,12 @@ static void* mistress_func(void* arg)
 	    if (strcmp(whf->sname, whf[1].sname)) continue;
 	    int	l = whf[1].segno - whf->segno;
 	    if (l == 1) { 
-		sprintf(str, "%s %d %d", whf->sname, whf->right, whf[1].left);
+		snprintf(str, LINE_MAX, "%s %d %d", 
+			whf->sname, whf->right, whf[1].left);
 		if ((*fsd)->getseq(str)) q->enqueue(fsd);
 	    } else if (l == -1) {
-		sprintf(str, "%s %d %d <", whf->sname, whf[1].left, whf->right);
+		snprintf(str, LINE_MAX, "%s %d %d <", 
+			whf->sname, whf[1].left, whf->right);
 		if ((*fsd)->getseq(str)) q->enqueue(fsd);
 	    }
 	}
@@ -1428,7 +1522,8 @@ const	char*	insuf = "No input seq file !\n";
 	    if (TgtMolc == PROTEIN && alprm2.spb > 0.) {
 		SeqServer	svr(argc, argv, IM_SNGL, 0, TgtMolc);
 		mb = makeblock(&svr);
-	    } else	mb = makeblock(argc, argv, molc, TgtMolc);
+	    } else
+		mb = makeblock(argc, argv, molc, TgtMolc);
 	    if (mb) {
 		mb->WriteBlkInfo();
 		mb->delete_dbf();
@@ -1448,8 +1543,7 @@ const	char*	insuf = "No input seq file !\n";
 	if (!algmode.mlt) OutPrm.MaxOut = 1;
 	if (OutPrm.MaxOut > OutPrm.MaxOut2) OutPrm.MaxOut2 = OutPrm.MaxOut;
 	setup_output(CDS_FORM, 0, false);	// output format
-	int	nseqs = no_seqs;
-	if (algmode.blk) nseqs += OutPrm.MaxOut2 + 1;
+	int	nseqs = no_seqs += OutPrm.MaxOut2 + 1;
 	Seq**	seqs = new Seq*[nseqs];
 	Seq*&	a = seqs[0];
 	Seq*&	b = seqs[1];
@@ -1479,21 +1573,23 @@ const	char*	dbs = genomedb? genomedb: (aadbs? aadbs: cdnadb);
 		messg = "Can't open query !\n";
 		goto postproc;
 	    }
-	    SrchBlk*	bprm = getblkinf(seqs, dbs, mb);
-	    delete mb;
-	    no_seqs = OutPrm.MaxOut2 + bprm->NoWorkSeq + 2;	// query + last
-	    if (a->inex.intr || b->inex.intr) makeStdSig53();
-	    set_max_extend_gene_rng(def_max_extend_gene_rng);
-	    if (bprm->pwd->DvsP == 3) OutPrm.SkipLongGap = 0;
-	    if (algmode.nsa == SAM_FORM) put_genome_entries();
 	    if (outputs.setup(a->spath)) {
+		SrchBlk*	bprm = getblkinf(seqs, dbs, mb);
+		delete mb;
+		no_seqs = OutPrm.MaxOut2 + bprm->NoWorkSeq + 2;	// query + last
+		if (a->inex.intr || b->inex.intr) makeStdSig53();
+		set_max_extend_gene_rng(def_max_extend_gene_rng);
+		if (bprm->pwd->DvsP == 3) OutPrm.SkipLongGap = 0;
+		if (algmode.nsa == SAM_FORM) put_genome_entries();
 #if M_THREAD
 		if (thread_num) MasterWorker(seqs, &svr, (void*) bprm); else
 #endif
 		all_in_func(seqs, &svr, (void*) bprm);
+		delete bprm;
 	    }
-	    delete bprm;
 	} else {
+	    extra_seq = svr.input_ns == 2? 0: 1;
+	    no_seqs = OutPrm.MaxOut2 + 2;
 	    if (svr.nextseq(b, 1) == IS_END) {
 		messg = "Can't open genomic sequence !\n";
 		goto postproc;
@@ -1503,8 +1599,20 @@ const	char*	dbs = genomedb? genomedb: (aadbs? aadbs: cdnadb);
 		goto postproc;
 	    }
 	    PwdB*	pwd = SetUpPwd(seqs);
-	    if (pwd->DvsP == 3) OutPrm.SkipLongGap = 0;
+	    b_intr = b->inex.intr;
 	    if (a->inex.intr || b->inex.intr) makeStdSig53();
+	    if (pwd->DvsP == 3) OutPrm.SkipLongGap = 0;
+	    else if (svr.input_ns == 1) {
+		INT	ori = q_mns? q_mns: 3;
+		Seq*&	c = seqs[2];
+		b->comrev(seqs + 2);
+		if (pwd->DvsP == 1) {
+		    b->nuc2tron();
+		    c->nuc2tron();
+		    if (!b->exin) b->exin = new Exinon(b, pwd, ori == 3);
+		    if (!c->exin) c->exin = new Exinon(c, pwd, ori == 3);
+		}
+	    }
 	    set_max_extend_gene_rng(0);
 	    if (outputs.setup(a->spath)) {
 #if M_THREAD
