@@ -6,7 +6,7 @@
 *	Saitama Cancer Center Research Institute
 *	818 Komuro, Ina-machi, Saitama 362-0806, Japan
 *
-*	Osamu Gotoh, Ph.D.	(2001-)
+*	Osamu Gotoh, Ph.D.	(2001-2023)
 *	National Institute of Advanced Industrial Science and Technology
 *	Computational Biology Research Center (CBRC)
 *	2-41-6 Aomi, Koutou-ku, Tokyo 135-0064, Japan
@@ -16,7 +16,8 @@
 *	Graduate School of Informatics, Kyoto University
 *	Yoshida Honmachi, Sakyo-ku, Kyoto 606-8501, Japan
 *
-*	Copyright(c) Osamu Gotoh <<o.gotoh@aist.go.jp>>
+*	Copyright(c) Osamu Gotoh <<gotoh.osamu.67a@st.kyoto-u.ac.jp>>
+*
 *****************************************************************************/
 
 #include "divseq.h"
@@ -69,7 +70,7 @@ static	const	char*	attrfrmt3 = " %d:%d  ( %d - %d )";
 static	const	char	grext[] = ".grd";		// gene
 static	const	char	erext[] = ".erd";		// exon
 static	const	char	qrext[] = ".qrd";		// query seq name
-static	const	char	arext[] = ".ard";		// alignment
+//static	const	char	arext[] = ".ard";	// alignment
 //	const	char	cigar[] = "MIDNSHP=XJA";	// J=5, A=3
 //	const	char	vulgar[] = "MCGN53ISFJA";	// J=5, A=3
 
@@ -408,7 +409,7 @@ const	RANGE*	wrng = rng = fistrng(rng);
 		    intv = i - r - 1;
 		    --wrng;
 		} else {
-		    if (mode == 2) sprintf(str + c, "%d..%d", l, r);
+		    if (mode == 2) snprintf(str + c, MAXL - c, "%d..%d", l, r);
 		    break;
 		}
 	    } else {
@@ -418,13 +419,13 @@ const	RANGE*	wrng = rng = fistrng(rng);
 		    intv = i - r - 1;
 		    ++wrng;
 		} else {
-		    if (mode == 2) sprintf(str + c, "%d..%d", l, r);
+		    if (mode == 2) snprintf(str + c, MAXL - c, "%d..%d", l, r);
 		    break;
 		}
 	    }
 	    if (intv == 0 || intv >= IntronPrm.llmt) {
 		if (mode != 2) continue;
-		sprintf(str + c, "%d..%d", l, r);
+		snprintf(str + c, MAXL - c, "%d..%d", l, r);
 		l = i;
 		strcat(str, ",");
 		if ((c = strlen(str)) > 56) {
@@ -492,8 +493,8 @@ const	char*	ltype;
 const	char*	fmt3;
 static	int	Gff3MID = 0;
 static	int	Gff3CID = 0;
-static	const	char*	fmt3g = "ID=gene%05d;Name=%s\n";
-static	const	char*	fmt3m = "ID=mRNA%05d;Parent=gene%05d;Name=%s\n";
+static	const	char*	fmt3g = "ID=gene%05d;Name=%s;Target=%s\n";
+static	const	char*	fmt3m = "ID=mRNA%05d;Parent=gene%05d;Name=%s;Target=%s\n";
 static	const	char*	fmt3c = "ID=%s%05d;Parent=mRNA%05d;Name=%s;";
 static	const	char*	fmt3t = "Target=%s %d %d %c\n";
 
@@ -507,7 +508,7 @@ static	const	char*	fmt3t = "Target=%s %d %d %c\n";
 	while (neoeij(wkr + 1)) ++wkr;
 	int	l = gene->SiteNo(rng->left);
 	int	r = gene->SiteNo(wkr->right - 1);
-	sprintf(mname, "%s_%d", (*gene->sname)[0], (l + r) / 2000);
+	snprintf(mname, MAXL, "%s_%d", (*gene->sname)[0], (l + r) / 2000);
 	if (!Gff3MID++) fputs("##gff-version\t3\n", fd);
 	l = gene->SiteNo(0);
 	r = gene->SiteNo(gene->len - 1);
@@ -518,10 +519,10 @@ static	const	char*	fmt3t = "Target=%s %d %d %c\n";
 	if (rvg == '-') swap(l, r);
 	fprintf(fd, fmt3n, (*gene->sname)[0], "gene", l, r, 
 	    (int) (scr / alprm.scale), rvg);
-	fprintf(fd, fmt3g, Gff3MID, mname);
+	fprintf(fd, fmt3g, Gff3MID, mname, (*qry->sname)[0]);
 	fprintf(fd, fmt3n, (*gene->sname)[0], "mRNA", l, r, 
 	    (int) (scr / alprm.scale), rvg);
-	fprintf(fd, fmt3m, Gff3MID, Gff3MID, mname);
+	fprintf(fd, fmt3m, Gff3MID, Gff3MID, mname, (*qry->sname)[0]);
 
 	wkr = rng;
 	prv = ++wsk;
@@ -578,7 +579,7 @@ static	const	char*	fmt3t = "Target=%s %d %d %c;Gap=";
 	while (neoeij(wkr + 1)) ++wkr;
 	int	l = gene->SiteNo(rng->left);
 	int	r = gene->SiteNo(wkr->right - 1);
-	sprintf(mname, "%s_%d", (*gene->sname)[0], (l + r) / 2000);
+	snprintf(mname, MAXL, "%s_%d", (*gene->sname)[0], (l + r) / 2000);
 	if (!Gff3MID++) fputs("##gff-version\t3\n", fd);
 	l = gene->SiteNo(0);
 	r = gene->SiteNo(gene->len - 1);
@@ -1367,7 +1368,8 @@ const	char*	sl = strrchr(spath, '/');
 
 	char	str[LINE_MAX];
 	strcpy(str, prefix);
-	char*	fn = str + strlen(str);
+	slen = strlen(str);
+	char*	fn = str + slen;
 	if (n_out_modes == 1) {
 	    out_mode[0] = algmode.nsa;
 	    if (OutPrm.out_file) {
@@ -1377,12 +1379,12 @@ const	char*	sl = strrchr(spath, '/');
 		        fds[0] = stdout;
 			return (1);
 		    }
-		    sprintf(fn, ".O%d", algmode.nsa);
+		    snprintf(fn, LINE_MAX - slen, ".O%d", algmode.nsa);
 		} else if (algmode.nsa == BIN_FORM) {
 		    fds[0] = stdout;
 		    return (1);
-		}
-		fds[0] = wfopen(str, "w");
+		} else 
+		    fds[0] = wfopen(OutPrm.out_file, "w");
 		if (!fds[0]) return (0);
 	    } else
 		fds[0] = stdout;
@@ -1394,7 +1396,7 @@ const	char*	sl = strrchr(spath, '/');
 	    if (out_mode[i] < 0) continue;
 	    if (out_mode[i] == BIN_FORM) fds[n] = 0;
 	    else {
-		sprintf(fn, ".O%d", i);
+		snprintf(fn, LINE_MAX - slen, ".O%d", i);
 		fds[n] = wfopen(str, "w");
 	    }
 	    if (fds[n]) out_mode[n++] = out_mode[i];
@@ -1471,7 +1473,7 @@ static int csym(const int cmp[], int rows, const SEQ_CODE* defcode)
 		if (cc < 0) cc = 0;
 		if (cc > nmk) cc = nmk;
 		return (mark[cc]);
-	    } else if (defcode->ceil_code == NTS)
+	    } else if (defcode->max_code == NSIMD)
 		return (logonuc(cmp, ii, defcode));
 	    else
 		return (chempro(cmp, ii));
@@ -1714,7 +1716,7 @@ static	const char* frm[] = {"%c%s [%d:%d] ", "%s ( %d - %d )",  "%-16s ( %3d - %
 	    fprintf(fd, frm[1], sqname(), SiteNo(left), SiteNo(right - 1));
 	} else {
 	    n = n == 3? 1: 2;
-	    sprintf(str, frm[0], senschar(), sqname(), many, len);
+	    snprintf(str, MAXL, frm[0], senschar(), sqname(), many, len);
 	    fprintf(fd, frm[n], str, SiteNo(left), SiteNo(right - 1));
 	}
 }
@@ -1905,16 +1907,16 @@ PrintMember::PrintMember(const Strlist* sn, bool pad_space, const char* tl)
 	if (OutPrm.taxoncode) {
 	    g2t = ftable.read_gnm2tab(OutPrm.taxoncode);
 	    if (pad_space)
-		sprintf(fmt, "%%s:%%-%ds", sname->longest());
+		snprintf(fmt, MAXL, "%%s:%%-%ds", sname->longest());
 	    else
 		strcpy(fmt, "%s:%s");
 	} else {
 	    if (pad_space)
-		sprintf(fmt, "%%-%ds", sname->longest());
+		snprintf(fmt, MAXL, "%%-%ds", sname->longest());
 	    else
 		strcpy(fmt, "%s");
 	}
-	if (tl) strcat(fmt, tl);
+	if (tl) strncat(fmt, tl, MAXL - strlen(fmt));
 }
 
 void PrintMember::put_member(FILE* fd, int i) const
@@ -1950,7 +1952,7 @@ PrintAln::PrintAln(const GAPS**  _gaps, Seq* _seqs[], const int& _seqnum, FILE*
 	}
 	if (markeij) {
 	    char	str[MAXL];
-	    sprintf(str, "%s: %s", seqnum == 1? "Prrn": "Aln", seqs[0]->sqname());
+	    snprintf(str, MAXL, "%s: %s", seqnum == 1? "Prrn": "Aln", seqs[0]->sqname());
 	    for (int n = 1; n < seqnum; ++n) {
 		strcat(str, " + ");
 		strcat(str, seqs[n]->sqname());
@@ -2164,16 +2166,16 @@ void PrintAln::printaln()
 	int 	active = seqnum;
 const	RANGE*	exon = 0;
 	int	maxleft = 0;
-	gene = -1;
+	gene = -1;	// htl 1: AvsA, 2: GvsC, 3: GvsA
 	for (int j = htl = 0; j < seqnum; ++j) {
 	    Seq*&	sd = seqs[j];
 	    if (sd->left > maxleft) maxleft = sd->left;
-	    if (sd->isprotein()) htl |= 1;
+	    if (sd->isprotein()) {htl |= 1; pro = j;}
+	    else	htl |= 2;
 	    if (sd->inex.intr && sd->exons) {
-		htl |= 2;		// 0: CvsC
 		exon = sd->exons + 1;	// 1: AvsA
-		gene = j;		// 2: GvsC
-		gpos = gaps[j]->gps;	// 3: GvsA
+		gene = j;
+		gpos = gaps[j]->gps;
 	    }
 	    for (int i = 0; i < sd->many; ++i, ++k, wbuf += OutPrm.lpw) {
 		nbr[k] = sd->calcnbr(gaps[j]->gps, i);
@@ -2184,7 +2186,6 @@ const	RANGE*	exon = 0;
 	    if (!gp[j]->gln) ++gp[j];
 	    agap[j] = 0;
 	}
-	pro = (gene >= 0)? gene + 1: -1;
 const	int	c_step = (htl == 1)? 3: 1;
 	if (markeij) {
 	    for (int j = 0; j < seqnum; ++j) {
@@ -2199,11 +2200,12 @@ const	int	c_step = (htl == 1)? 3: 1;
 	    }
 	}
 
-	int	z = 0, gph = 0, phs = 0, intlen = 0;
+	int	z = 0, phs = 0, intlen = 0;
+	int	df = 0;				// df >=0? CDS: intron
 	if (htl & 2) prmode = Row_None;
 	do {
 	    if (OutPrm.SkipLongGap) {
-		int	jj = 0;			//	skip long insert
+		int	jj = 0;			// skip long insert
 		int	gap = INT_MAX;
 		for (int j = 0; j < seqnum; ++j) {
 		    int	upr = gp[j]->gps - gaps[j]->gps;
@@ -2220,7 +2222,7 @@ const		int	upr = (gap - z - OutPrm.EijMergin) / OutPrm.lpw * OutPrm.lpw;
 			Seq*&	sd = seqs[j];
 const			bool	step3 = htl == 3 && !sd->inex.intr;
 			if (j == jj) {
-			    if (step3) gph = phs = (phs + upr) % 3;
+			    if (step3) phs = (phs + upr) % 3;
 			} else {
 			    int	inc = upr;
 			    if (step3) {
@@ -2246,23 +2248,27 @@ const			bool	step3 = htl == 3 && !sd->inex.intr;
 	    for (int clm = 0; clm < OutPrm.lpw; ++clm, ++z, ++gpos, cpos += c_step, pphs = phs) {
 		int	reij = 0;
 		if (exon) {
-		    while (exon->right <= gpos) {intlen = exon[1].left - exon->right; ++exon;}
+		    while (exon->right <= gpos) {	// skip leading exons
+			intlen = exon[1].left - exon->right;
+			++exon;
+		    }
+		    df = gpos - exon->left;
 		    if (exon->right == gpos + 1 && phs == 1) reij = 3;
 		}
 		if (active) active = seqnum;
 		for (int j = k = 0; j < seqnum; k += seqs[j++]->many) {
 		    Seq*&	sd = seqs[j];
-		    bool	prot = sd->isprotein() && htl == 3;
+const		    bool	prot = sd->isprotein() && htl == 3;
 	loop:	    int		pos = gp[j]->gps - gaps[j]->gps;
-		    bool	neog = gaps_intr(gp[j]);
-		    int	gap = neog? gp[j]->gln: 0;
+const		    bool	neog = gaps_intr(gp[j]);
+const		    int	gap = neog? gp[j]->gln: 0;
 		    if (!active) {
 			for (int i = 0; i < sd->many; ++i)
 			    image[k+i][clm] = BLANK;
 		    } else if (pos > z) {
 			if (prot) {
 			    for (int i = 0; i < sd->many; ++i) {
-				if (phs == 1) {
+				if (phs == 1 && df >= 0) {
 				    image[k+i][clm] = *wkr[j] == nil_code? gap_code: *wkr[j];
 				    if (IsntGap(*wkr[j])) ++nbr[k+i];
 				    ++wkr[j];
@@ -2273,7 +2279,6 @@ const			bool	step3 = htl == 3 && !sd->inex.intr;
 			    for (int i = 0; i < sd->many; ++i) {
 				image[k+i][clm] = *wkr[j] == nil_code? gap_code: *wkr[j];
 				if (j == gene) {
-				    int	df = gpos - exon->left;
 				    if (df < 0) image[k+i][clm] |= INTRONBIT;
 				    else if (intlen > 2 && df < 2 && (htl & 2) && pphs == 1)
 					reij = 2 - df;
@@ -2284,7 +2289,8 @@ const			bool	step3 = htl == 3 && !sd->inex.intr;
 				++wkr[j];
 			    }
 			}
-			if ((htl & 2) && j == pro) gph = phs = (phs + 1) % 3;
+			if ((htl & 2) && j == pro && df >= 0)
+			    phs = next_p[phs];
 			if (markeij && pfqs[j]) {
 			    int	niis = 0;
 			    while ((pfqs[j] + niis < tfqs[j]) &&
@@ -2296,7 +2302,7 @@ const			bool	step3 = htl == 3 && !sd->inex.intr;
 		    } else if (pos + gap > z || !neog) {
 			if (prot) {
 			    for (int i = 0; i < sd->many; i++) {
-				if (gph == 1 && (!exon || gpos > exon->left))
+				if (phs == 1 && (!exon || gpos > exon->left))
 				    image[k+i][clm] = gap_code;
 				else
 				    image[k+i][clm] = BLANK;
@@ -2306,7 +2312,8 @@ const			bool	step3 = htl == 3 && !sd->inex.intr;
 				image[k+i][clm] = (!exon || gpos >= exon->left)?
 				    gap_code: BLANK;
 			}
-			if (htl & 2) gph = (gph + 1) % htl;
+			if ((htl & 2) && j == pro && df >= 0)
+			    phs = next_p[phs];
 			if (!neog && !--active && !clm--) return;
 		    } else {
 			agap[j] += gp[j]->gln * c_step;
@@ -2330,14 +2337,14 @@ void pralnseq(const GAPS** gaps, Seq* seqs[], const int& seqnum, FILE* fd)
 static void put_SigII(const PFQ* pfq, const PFQ* tfq, const int* lst, 
 	int numlst, FILE* fd)
 {
-	int	lwd = OutPrm.lpw >= 10? OutPrm.lpw - 4: 56;
+const	int	lwd = OutPrm.lpw >= 10? OutPrm.lpw - 4: 56;
 	char	str[MAXL];
 
 	fprintf(fd, ";B %ld %d\n", (long) (tfq - pfq), numlst);
 	int	c = 0;
 	for ( ; pfq < tfq - 1; ++pfq) {
 	    if (c == 0) fputs(";b", fd);
-	    sprintf(str + c, " %d %d,", pfq->pos, pfq->num);
+	    snprintf(str + c, MAXL - c, " %d %d,", pfq->pos, pfq->num);
 	    if ((c = strlen(str)) > lwd) {
 		fputs(str, fd);
 		fputs("\n", fd);
@@ -2348,10 +2355,10 @@ static void put_SigII(const PFQ* pfq, const PFQ* tfq, const int* lst,
 	else	fputs(";b", fd);
 	fprintf(fd, " %d %d\n", pfq->pos, pfq->num);
 	if (!lst || !numlst) return;
-	const int*	tst = lst + numlst;
+const	int*	tst = lst + numlst;
 	for (c = 0; lst < tst - 1; ++lst) {
 	    if (c == 0) fputs(";m", fd);
-	    sprintf(str + c, " %d", *lst + 1);
+	    snprintf(str + c, MAXL - c, " %d", *lst + 1);
 	    if ((c = strlen(str)) > lwd) {
 		fputs(str, fd);
 		fputs("\n", fd);
@@ -2372,7 +2379,7 @@ const	int*	lst = sigII? sigII->lst: 0;
 	    fputs(";B 0 0\n", fd);
 	    return;
 	}
-	int	step = isprotein()? 3: 1;
+const	int	step = isprotein()? 3: 1;
 	int	upto = step * (left + base_);
 	for ( ; pfq->pos < upto; ++pfq)
 	    if (lst) lst += pfq->num;
@@ -2399,13 +2406,13 @@ static void nexus_head(FILE* fd, const Seq* sd)
 	fputs("\tmatrix\n", fd);
 }
 
-void Seq::typeseq(FILE* fd, bool in_line)
+void Seq::typeseq(FILE* fd, const bool& in_line)
 {
 	if (!many) return;
 	if (!fd) fd = out_fd;
 	if (in_line) {
-	    CHAR*	t = at(right);
-	    for (CHAR*	p = at(left); p < t; ++p) 
+const	    CHAR*	t = at(right);
+	    for (const CHAR* p = at(left); p < t; ++p) 
 		putc(code->decode[*p], fd);
 	} else if (many == 1) {
 	    listseq(fd);
